from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE


class RDSInstanceAutoBackupEncryptionWithCMK(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure replicated backups are encrypted at rest using KMS CMKs"
        id = "CKV_AWS_245"
        supported_resources = ['aws_db_instance_automated_backups_replication']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        """
            Looks for encryption configuration for backup replication:
            https://registry.terraform.io/providers/hashicorp/aws/latest/docs/resources/db_instance_automated_backups_replication
        :param conf: aws_db_instance_automated_backups_replication configuration
        :return: <CheckResult>
        """
        return 'kms_key_id'

    def get_expected_value(self):
        return ANY_VALUE


check = RDSInstanceAutoBackupEncryptionWithCMK()



from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AzureServicebusDoubleEncryptionEnabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Azure Service Bus uses double encryption"
        id = "CKV_AZURE_199"
        supported_resources = ("azurerm_servicebus_namespace",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "customer_managed_key/[0]/infrastructure_encryption_enabled"

    def get_expected_value(self) -> Any:
        return True


check = AzureServicebusDoubleEncryptionEnabled()



from __future__ import annotations

import logging
import os
import sys

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.output.report import Report
from checkov.common.runners.base_runner import BaseRunner
from checkov.common.sast.consts import SUPPORT_FILE_EXT, FILE_EXT_TO_SAST_LANG
from checkov.runner_filter import RunnerFilter
from checkov.sast.checks_infra.base_registry import Registry
from checkov.sast.engines.prisma_engine import PrismaEngine

from typing import List, Optional

logger = logging.getLogger(__name__)


class Runner(BaseRunner[None, None, None]):
    check_type = CheckType.SAST  # noqa: CCE003  # a static attribute

    def __init__(self) -> None:
        super().__init__(file_extensions=["." + a for a in FILE_EXT_TO_SAST_LANG.keys()])
        self.registry = Registry()
        self.engine = PrismaEngine()  # noqa: disallow-untyped-calls

    def should_scan_file(self, file: str) -> bool:
        for extensions in SUPPORT_FILE_EXT.values():
            for extension in extensions:
                if file.endswith(extension):
                    return True
        return False

    def run(self, root_folder: Optional[str],
            external_checks_dir: Optional[List[str]] = None,
            files: Optional[List[str]] = None,
            runner_filter: Optional[RunnerFilter] = None,
            collect_skip_comments: bool = True) -> List[Report]:

        if sys.platform.startswith('win'):
            # TODO: Enable SAST for windows runners
            return [Report(self.check_type)]

        if not runner_filter:
            logger.warning('no runner filter')
            return [Report(self.check_type)]

        if bc_integration.daemon_process:
            # only happens for 'ParallelizationType.SPAWN'
            bc_integration.setup_http_manager()
            bc_integration.set_s3_client()

        # registry get all the paths
        self.registry.set_runner_filter(runner_filter)
        self.registry.add_external_dirs(external_checks_dir)

        targets = []
        if root_folder:
            if not os.path.isabs(root_folder):
                root_folder = os.path.abspath(root_folder)
            targets.append(root_folder)
        if files:
            targets.extend([a if os.path.isabs(a) else os.path.abspath(a) for a in files])

        reports = []
        try:
            reports = self.engine.get_reports(targets, self.registry, runner_filter.sast_languages)
        except BaseException as e:
            logger.error(f"got error when try to run prisma sast: {e}")

        return reports



from __future__ import annotations

import hashlib
import logging
import os
import platform
from typing import TYPE_CHECKING, Optional, List, Tuple

from detect_secrets.core import scan

from checkov.common.util.stopit import ThreadingTimeout, SignalTimeout, TimeoutException
from checkov.common.util.decorators import time_it
from checkov.common.parallelizer.parallel_runner import parallel_runner
from checkov.common.secrets.consts import GIT_HISTORY_NOT_BEEN_REMOVED
from checkov.secrets.git_history_store import GitHistorySecretStore, RawStore, RENAME_STR, FILE_RESULTS_STR
from checkov.secrets.git_types import Commit, CommitMetadata, PROHIBITED_FILES

if TYPE_CHECKING:
    from detect_secrets import SecretsCollection

os.environ["GIT_PYTHON_REFRESH"] = "quiet"
try:
    from git import Repo, Tree  # type: ignore

    git_import_error = None
except ImportError as e:
    git_import_error = e

MIN_SPLIT = 100
FILES_TO_IGNORE_IN_GIT_HISTORY = ('.md', '.svg', '.png', '.jpg') + PROHIBITED_FILES


class GitHistoryScanner:
    commits_count = 0  # noqa: CCE003

    def __init__(self, root_folder: str, secrets: SecretsCollection,
                 history_store: Optional[GitHistorySecretStore] = None, timeout: int = 43200):
        """
        root_folder: Is necessary for initializing the Repo to read from
        secrets: An object which will be filled with secrets during the run of the secrets scan
        history_store: A helper objects which will be field during the run, to map between found secrets and commits.
            is not used afterwards for outside-of-class work
        """
        self.root_folder = root_folder
        self.secrets = secrets
        self.timeout = timeout
        # in case we start from mid-history (git) we want to continue from where we've been
        self.history_store: GitHistorySecretStore = history_store or GitHistorySecretStore()
        self.raw_store: List[RawStore] = []
        self.repo: Repo

    def scan_history(self, last_commit_scanned: Optional[str] = '') -> bool:
        """return true if the scan finished without timeout"""
        is_repo_set = self.set_repo()  # for mocking purposes in testing
        if not is_repo_set:
            logging.info("Couldn't set git repo. Cannot proceed with git history scan.")
            return False
        timeout_class = ThreadingTimeout if platform.system() == 'Windows' else SignalTimeout
        # mark the scan to finish within the timeout
        with timeout_class(self.timeout) as to_ctx_mgr:
            scanned = self._scan_history(last_commit_scanned)
            self._create_secret_collection()
        if to_ctx_mgr.state == to_ctx_mgr.TIMED_OUT:
            logging.info(f"timeout reached ({self.timeout}), stopping scan.")
            return False
        # else: everything was OK
        return scanned

    def _scan_history(self, last_commit_scanned: Optional[str] = '') -> bool:
        commits_diff: List[Commit] = []
        if not last_commit_scanned:
            first_commit_diff = self._get_first_commit()
            if first_commit_diff:
                commits_diff.append(self._get_first_commit())
        commits_diff.extend(self._get_commits_diff(last_commit_sha=last_commit_scanned))
        logging.info(f"[_scan_history] got {len(commits_diff)} files diffs in {self.commits_count} commits")
        if self.commits_count > MIN_SPLIT:
            logging.info("[_scan_history] starting parallel scan")
            self._run_scan_parallel(commits_diff)
        else:
            logging.info("[_scan_history] starting single scan")
            self.raw_store.extend(self._run_scan_one_bulk(commits_diff))

        if not self.raw_store:  # scanned nothing
            return False

        self._process_raw_store()
        return True

    def _process_raw_store(self) -> None:
        for raw_res in self.raw_store:
            res_type = raw_res.get('type')
            if res_type == FILE_RESULTS_STR:
                self.history_store.set_secret_map(raw_res.get('file_results', []), raw_res.get('file_name', ''),
                                                  raw_res['commit'])
            elif res_type == RENAME_STR:
                self.history_store.handle_renamed_file(raw_res.get('rename_from', ''), raw_res.get('rename_to', ''),
                                                       raw_res['commit'])

    def _create_secret_collection(self) -> None:
        # run over the entire history store and create the secret collection
        for secrets_data in self.history_store.secrets_by_file_value_type.values():
            for secret_data in secrets_data:
                removed = secret_data["removed_commit_hash"] if secret_data[
                    "removed_commit_hash"] else GIT_HISTORY_NOT_BEEN_REMOVED
                key = f'{secret_data["added_commit_hash"]}_{removed}_{secret_data["potential_secret"].filename}'
                self.secrets[key].add(secret_data["potential_secret"])
        logging.info(f"Created secret collection for {len(self.history_store.secrets_by_file_value_type)} secrets")

    def set_repo(self, root_folder: str | None = None) -> bool:
        if not root_folder:
            root_folder = self.root_folder
        if git_import_error is not None:
            logging.warning(f"Unable to load git module (is the git executable available?) {git_import_error}")
            return False
        try:
            self.repo = Repo(root_folder)
            return True
        except Exception as e:
            logging.error(f"Folder {root_folder} is not a GIT project {e}")
            return False

    def _get_commits_diff(self, last_commit_sha: Optional[str] = None) -> List[Commit]:
        """
        :param: last_commit_sha = is the last commit we have already scanned. in case it exist the function will
        return the commits from the revision of param to the current head
        """
        logging.info("[_get_commits_diff] started")
        if last_commit_sha:
            curr_rev = self.repo.head.commit.hexsha
            commits = list(self.repo.iter_commits(last_commit_sha + '..' + curr_rev))
        else:
            commits = list(self.repo.iter_commits(self.repo.active_branch))
        GitHistoryScanner.commits_count = len(commits)
        commits_diff: List[Commit] = []
        for previous_commit_idx in range(GitHistoryScanner.commits_count - 1, 0, -1):
            try:
                current_commit_idx = previous_commit_idx - 1
                current_commit_hash = commits[current_commit_idx].hexsha
                committed_datetime: str = commits[current_commit_idx].committed_datetime.isoformat()
                committer: str = commits[current_commit_idx].committer.name or ''
                git_diff = commits[previous_commit_idx].diff(current_commit_hash, create_patch=True)
                curr_diff: Commit = Commit(
                    metadata=CommitMetadata(
                        commit_hash=current_commit_hash,
                        committer=committer,
                        committed_datetime=committed_datetime
                    )
                )
                for file_diff in git_diff:
                    file_name = file_diff.a_path if file_diff.a_path else file_diff.b_path
                    if file_name.endswith(FILES_TO_IGNORE_IN_GIT_HISTORY):
                        continue
                    file_path = os.path.join(self.root_folder, file_name)

                    if file_diff.renamed_file:
                        logging.debug(f"File was renamed from {file_diff.rename_from} to {file_diff.rename_to}")
                        curr_diff.rename_file(
                            file_path=file_path,
                            prev_filename=file_diff.rename_from,
                            new_filename=file_diff.rename_to
                        )
                        continue

                    elif file_diff.deleted_file:
                        logging.debug(f"File {file_diff.a_path} was deleted")

                    base_diff_format = f'diff --git {self.root_folder}/{file_diff.a_path} {self.root_folder}/{file_diff.b_path}' \
                                       f'\nindex 0000..0000 0000\n--- {self.root_folder}/{file_diff.a_path}\n+++ {self.root_folder}/{file_diff.b_path}\n'
                    curr_diff.add_file(filename=file_path, commit_diff=base_diff_format + self.get_decoded_diff(file_diff.diff))
                if not curr_diff.is_empty():
                    commits_diff.append(curr_diff)
            except TimeoutException:
                logging.error(f"stopped while getting commits diff, iteration: {previous_commit_idx}")
                return []
            except Exception as err:
                logging.warning(f"got error while getting commits diff, iteration: {previous_commit_idx}, error: {err}")
                continue
        logging.info("[_get_commits_diff] ended")
        return commits_diff

    def _run_scan_parallel(self, commits_diff: List[Commit]) -> None:
        results = parallel_runner.run_function(GitHistoryScanner._run_scan_one_bulk, commits_diff)

        for result in results:
            if result:
                self.raw_store.extend(result)

    @staticmethod
    def _run_scan_one_bulk(commits_diff: List[Commit] | Commit) -> List[RawStore]:
        scanned_file_count = 0
        results: List[RawStore] = []
        # parallel runner can make the list flat, so I can get here dict instead of list
        if isinstance(commits_diff, Commit):
            results, scanned_file_count = GitHistoryScanner._run_scan_one_commit(commits_diff)
        elif isinstance(commits_diff, list):
            for commit in commits_diff:
                cur_results, curr_count = GitHistoryScanner._run_scan_one_commit(commit)
                scanned_file_count += curr_count
                results.extend(cur_results)
        logging.debug(f"Scanned {scanned_file_count} git history files")
        return results

    @staticmethod
    def _run_scan_one_commit(commit: Commit) -> Tuple[List[RawStore], int]:
        results: List[RawStore] = []
        scanned_file_count = 0
        commit_hash = commit.metadata.commit_hash
        for file_name, file_diff in commit.files.items():
            file_results = [*scan.scan_diff(file_diff)]
            if file_results:
                logging.debug(
                    f"Found {len(file_results)} secrets in file path {file_name} in commit {commit_hash}")
                results.append(RawStore(file_results=file_results, file_name=file_name, commit=commit,
                                        type=FILE_RESULTS_STR, rename_from='', rename_to=''))
        for _, details in commit.renamed_files.items():
            rename_from = details['rename_from']
            rename_to = details['rename_to']
            results.append(RawStore(file_results=[], file_name='', commit=commit, type=RENAME_STR,
                                    rename_from=rename_from, rename_to=rename_to))
            scanned_file_count += 1
        return results, scanned_file_count

    @time_it
    def _get_first_commit(self) -> Commit:
        first_commit_sha = self.repo.git.log('--format=%H', '--max-parents=0', 'HEAD').split()[0]
        first_commit = self.repo.commit(first_commit_sha)
        empty_tree_sha = bytes.fromhex(hashlib.sha1(b'tree 0\0').hexdigest())  # nosec
        empty_tree = Tree(self.repo, empty_tree_sha)
        git_diff = empty_tree.diff(first_commit, create_patch=True)

        first_commit_diff: Commit = Commit(
            metadata=CommitMetadata(
                commit_hash=first_commit.hexsha,
                committer=first_commit.committer.name or '',
                committed_datetime=first_commit.committed_datetime.isoformat()
            )
        )

        for file_diff in git_diff:
            file_name = file_diff.b_path
            if file_name.endswith(FILES_TO_IGNORE_IN_GIT_HISTORY):
                continue
            file_path = os.path.join(self.root_folder, file_name)
            base_diff_format = f"--- ''\n+++ {file_path}\n"
            full_diff_format = base_diff_format + self.get_decoded_diff(file_diff.diff)
            first_commit_diff.add_file(filename=file_path, commit_diff=full_diff_format)
        return first_commit_diff

    @staticmethod
    def get_decoded_diff(diff: bytes) -> str:
        if diff is None:
            return ''
        try:
            decoded_diff = diff.decode('utf-8')
        except UnicodeDecodeError as ue:
            logging.debug(f'failed decoding file diff, {ue}')
            decoded_diff = diff.decode('utf-8', errors='ignore')
        return decoded_diff



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class StateMachineXray(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure State Machine has X-Ray tracing enabled"
        id = "CKV_AWS_284"
        supported_resources = ['aws_sfn_state_machine']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "tracing_configuration/[0]/enabled"


check = StateMachineXray()



from __future__ import annotations

import logging
import os
from typing import Optional, List, Tuple, Dict, Any

from termcolor import colored

from checkov.common.models.enums import CheckResult
from checkov.common.secrets.consts import ValidationStatus, GIT_HISTORY_NOT_BEEN_REMOVED

from checkov.common.bridgecrew.severities import Severity

from checkov.common.output.record import Record
from checkov.common.typing import _CheckResult

COMMIT_ADDED_STR = 'Commit Added'
COMMIT_REMOVED_STR = 'Commit Removed'

WARNING_SIGN_UNICODE = '\u26a0'
TEXT_BY_SECRET_VALIDATION_STATUS = {
    ValidationStatus.VALID.value: colored(f'\t{WARNING_SIGN_UNICODE} This secret has been validated'
                                          f' and should be prioritized', "red"),
    ValidationStatus.INVALID.value: colored('\tThis is not a valid secret and can be de-prioritized', "white"),
    ValidationStatus.UNKNOWN.value: '\tWe were not able to validate this secret',
    ValidationStatus.UNAVAILABLE.value: ''
}


class SecretsRecord(Record):
    def __init__(self,
                 check_id: str,
                 check_name: str,
                 check_result: _CheckResult,
                 code_block: List[Tuple[int, str]],
                 file_path: str,
                 file_line_range: List[int],
                 resource: str,
                 evaluations: Optional[Dict[str, Any]],
                 check_class: str,
                 file_abs_path: str,
                 entity_tags: Optional[Dict[str, str]] = None,
                 caller_file_path: Optional[str] = None,
                 caller_file_line_range: Optional[Tuple[int, int]] = None,
                 bc_check_id: Optional[str] = None,
                 resource_address: Optional[str] = None,
                 severity: Optional[Severity] = None,
                 bc_category: Optional[str] = None,
                 benchmarks: dict[str, list[str]] | None = None,
                 description: Optional[str] = None,
                 short_description: Optional[str] = None,
                 vulnerability_details: Optional[Dict[str, Any]] = None,
                 connected_node: Optional[Dict[str, Any]] = None,
                 details: Optional[List[str]] = None,
                 check_len: int | None = None,
                 definition_context_file_path: Optional[str] = None,
                 validation_status: Optional[str] = None,
                 added_commit_hash: Optional[str] = None,
                 removed_commit_hash: Optional[str] = None,
                 added_by: Optional[str] = None,
                 removed_date: Optional[str] = None,
                 added_date: Optional[str] = None
                 ):
        super().__init__(check_id=check_id,
                         check_name=check_name,
                         check_result=check_result,
                         code_block=code_block,
                         file_path=file_path,
                         file_line_range=file_line_range,
                         resource=resource,
                         evaluations=evaluations,
                         check_class=check_class,
                         file_abs_path=file_abs_path,
                         entity_tags=entity_tags,
                         bc_check_id=bc_check_id,
                         severity=severity,
                         details=details,
                         caller_file_path=caller_file_path,
                         caller_file_line_range=caller_file_line_range,
                         resource_address=resource_address,
                         bc_category=bc_category,
                         benchmarks=benchmarks,
                         description=description,
                         short_description=short_description,
                         vulnerability_details=vulnerability_details,
                         connected_node=connected_node,
                         check_len=check_len,
                         definition_context_file_path=definition_context_file_path
                         )
        self.validation_status = validation_status
        self.added_commit_hash = added_commit_hash
        self.removed_commit_hash = removed_commit_hash
        self.added_by = added_by
        self.removed_date = removed_date
        self.added_date = added_date

    def to_string(self, compact: bool = False, use_bc_ids: bool = False) -> str:
        processed_record = super().to_string(compact=compact, use_bc_ids=use_bc_ids)
        validation_status_message = self._get_secret_validation_status_message()
        if validation_status_message and self.check_result["result"] == CheckResult.FAILED and os.getenv("CKV_VALIDATE_SECRETS"):
            # if needed insert validation status message
            splitted_record = processed_record.split("\n")
            splitted_record.insert(2, validation_status_message)
            processed_record = "\n".join(splitted_record)

        processed_record = self._add_commit_details(processed_record)
        return processed_record

    def _add_commit_details(self, processed_record: str) -> str:
        if not self.added_commit_hash and not self.is_empty_removed_commit():
            return processed_record
        splitted_record = processed_record.split("\n")
        file_idx = 0
        file_line = ''
        for idx, line in enumerate(splitted_record):
            if line.__contains__('File:'):
                file_idx = idx
                file_line = line
                break
        added = False
        if self.added_commit_hash:
            file_line = file_line + f'; {COMMIT_ADDED_STR}: {self.added_commit_hash}'
            added = True
        if self.removed_commit_hash:
            file_line = file_line + f'; {COMMIT_REMOVED_STR}: {self.removed_commit_hash}'
            added = True
        if added:
            splitted_record[file_idx] = file_line
            processed_record = "\n".join(splitted_record) + '\n'
        return processed_record

    def is_empty_removed_commit(self) -> bool:
        return (not self.removed_commit_hash) or (self.removed_commit_hash == GIT_HISTORY_NOT_BEEN_REMOVED)

    def _get_secret_validation_status_message(self) -> str:
        message = None
        if self.validation_status:
            message = TEXT_BY_SECRET_VALIDATION_STATUS.get(self.validation_status)

            if not message and self.validation_status != ValidationStatus.UNAVAILABLE.value:
                logging.debug(f'Got empty message for secret validation status = {self.validation_status}')

        return message or ''





from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class SSMDocumentsArePrivate(BaseResourceNegativeValueCheck):
    def __init__(self):
        """
        NIST.800-53.r5 AC-21, NIST.800-53.r5 AC-3, NIST.800-53.r5 AC-3(7), NIST.800-53.r5 AC-4, NIST.800-53.r5 AC-4(21),
        NIST.800-53.r5 AC-6, NIST.800-53.r5 SC-7, NIST.800-53.r5 SC-7(11), NIST.800-53.r5 SC-7(16),
        NIST.800-53.r5 SC-7(20), NIST.800-53.r5 SC-7(21), NIST.800-53.r5 SC-7(3), NIST.800-53.r5 SC-7(4),
        NIST.800-53.r5 SC-7(9)
        """
        name = "Ensure SSM documents are not Public"
        id = "CKV_AWS_303"
        supported_resources = ['aws_ssm_document']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "permissions/[0]/account_ids"

    def get_forbidden_values(self) -> str:
        return "All"


check = SSMDocumentsArePrivate()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class ACRContainerScanEnabled(BaseResourceCheck):
    SKUS = {"Standard", "Premium"}  # noqa: CCE003  # a static attribute

    def __init__(self):
        name = "Enable vulnerability scanning for container images."
        id = "CKV_AZURE_163"
        supported_resources = ("azurerm_container_registry",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        # 'ContainerRegistry' tier
        # 'standard', or higher -
        # not Basic
        if (
            "sku" in conf.keys()
            and isinstance(conf["sku"][0], str)
            and conf["sku"][0] in ACRContainerScanEnabled.SKUS
        ):
            return CheckResult.PASSED

        return CheckResult.FAILED


check = ACRContainerScanEnabled()



from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class WinVMEncryptionAtHost(BaseResourceValueCheck):
    def __init__(self) -> None:
        """
        If enabled, all the disks (including the temp disk) attached to this Virtual Machine will be encrypted

        If not enabled:
        https://learn.microsoft.com/en-gb/azure/virtual-machines/disks-enable-host-based-encryption-portal?tabs=azure-cli#prerequisites

        """
        name = "Ensure Windows VM enables encryption"
        id = "CKV_AZURE_151"
        supported_resources = ("Microsoft.Compute/virtualMachines",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "properties/securityProfile/encryptionAtHost"


check = WinVMEncryptionAtHost()



from checkov.arm.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories
from checkov.common.models.enums import CheckResult


class KeyVaultEnablesSoftDelete(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that key vault enables soft delete"
        id = "CKV_AZURE_111"
        supported_resources = ['Microsoft.KeyVault/vaults']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self) -> str:
        return "properties/enableSoftDelete"


check = KeyVaultEnablesSoftDelete()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.common.util.data_structures_utils import find_in_dict
from checkov.kubernetes.checks.resource.base_spec_check import BaseK8Check
from checkov.common.util.type_forcers import force_list


class Seccomp(BaseK8Check):

    def __init__(self) -> None:
        # CIS-1.5 5.7.2
        name = "Ensure that the seccomp profile is set to docker/default or runtime/default"
        id = "CKV_K8S_31"
        # Location: Pod.metadata.annotations.seccomp.security.alpha.kubernetes.io/pod
        # Location: CronJob.spec.jobTemplate.spec.template.metadata.annotations.seccomp.security.alpha.kubernetes.io/pod
        # Location: *.spec.template.metadata.annotations.seccomp.security.alpha.kubernetes.io/pod
        # Location: *.spec.securityContext.seccompProfile.type
        supported_kind = ('Pod', 'Deployment', 'DaemonSet', 'StatefulSet', 'ReplicaSet', 'ReplicationController', 'Job', 'CronJob')
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def scan_spec_conf(self, conf: dict[str, Any]) -> CheckResult:
        metadata = {}

        if conf['kind'] == 'Pod':
            security_profile = find_in_dict(conf, 'spec/securityContext/seccompProfile/type')
            if security_profile:
                return CheckResult.PASSED if security_profile == 'RuntimeDefault' else CheckResult.FAILED
            if "metadata" in conf:
                metadata = conf["metadata"]
            if "spec" in conf and isinstance(conf["spec"], dict):
                containers = conf["spec"].get("containers")
                if containers:
                    containers = force_list(containers)
                    num_containers = len(containers)
                    passed_containers = 0
                    for container in containers:
                        security_profile = find_in_dict(container, "securityContext/seccompProfile/type")
                        if security_profile:
                            if security_profile == "RuntimeDefault":
                                passed_containers += 1
                            else:
                                return CheckResult.FAILED
                    if passed_containers == num_containers:
                        return CheckResult.PASSED

        if conf['kind'] in ['Deployment', 'StatefulSet', 'DaemonSet', 'Job', 'ReplicaSet']:
            security_profile = find_in_dict(conf, 'spec/template/spec/securityContext/seccompProfile/type')
            if security_profile:
                return CheckResult.PASSED if security_profile == 'RuntimeDefault' else CheckResult.FAILED

            metadata = find_in_dict(input_dict=conf, key_path="spec/template/metadata")
            if not metadata and "metadata" in conf:
                metadata = conf["metadata"]
        elif conf['kind'] == 'CronJob':
            inner_template = find_in_dict(input_dict=conf, key_path="spec/jobTemplate/spec/template")
            if inner_template and isinstance(inner_template, dict):
                if "metadata" in inner_template:
                    metadata = inner_template["metadata"]
                elif "spec" in inner_template:
                    inner_spec = inner_template["spec"]
                    if "metadata" in inner_spec:
                        metadata = inner_spec["metadata"]
                    elif "securityContext" in inner_spec:
                        security_profile = inner_spec["securityContext"].get("seccompProfile", {}).get("type")
                        return CheckResult.PASSED if security_profile == 'RuntimeDefault' else CheckResult.FAILED
        else:
            inner_metadata = find_in_dict(input_dict=conf, key_path="spec/template/metadata")
            metadata = inner_metadata if inner_metadata else metadata

        if metadata:
            if metadata.get('annotations'):
                for annotation in force_list(metadata["annotations"]):
                    for key in annotation:
                        if "seccomp.security.alpha.kubernetes.io/pod" in key:
                            if "docker/default" in annotation[key] or "runtime/default" in annotation[key]:
                                return CheckResult.PASSED
        return CheckResult.FAILED


check = Seccomp()



from __future__ import annotations

from typing import Any
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.checks.enums import BlockType
from checkov.openapi.checks.resource.v2.BaseOpenapiCheckV2 import BaseOpenapiCheckV2


class SecurityRequirement(BaseOpenapiCheckV2):
    def __init__(self) -> None:
        id = "CKV_OPENAPI_6"
        name = "Ensure that security requirement defined in securityDefinitions - version 2.0 files"
        categories = [CheckCategories.API_SECURITY]
        supported_resources = ["security"]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_resources,
            block_type=BlockType.DOCUMENT,
        )

    def scan_openapi_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]]:
        self.evaluated_keys = ["securityDefinitions"]
        if "securityDefinitions" not in conf:
            return CheckResult.FAILED, conf

        security_definitions = conf["securityDefinitions"]
        if not self.check_security_conf(conf, security_definitions):
            return CheckResult.FAILED, conf["security"]

        if "paths" not in conf:
            return CheckResult.FAILED, conf
        paths = conf["paths"]
        if not isinstance(paths, dict):
            return CheckResult.FAILED, conf

        for path, http_method in paths.items():
            if self.is_start_end_line(path) or not http_method:
                continue
            for op_name, op_val in http_method.items():
                if self.is_start_end_line(op_name):
                    continue
                if not isinstance(op_val, dict):
                    return CheckResult.FAILED, conf
                if not self.check_security_conf(op_val, security_definitions):
                    return CheckResult.FAILED, op_val["security"]

        return CheckResult.PASSED, conf

    def check_security_conf(self, conf: dict[str, Any], security_definitions: dict[str, Any]) -> bool:
        self.evaluated_keys = ["security"]
        return not (
            "security" in conf
            and conf["security"]
            and not self.is_requirements_defined(conf["security"], security_definitions)
        )

    def is_requirements_defined(self, security: list[dict[str, Any]], security_definitions: dict[str, Any]) -> bool:
        for scheme in security:
            if not isinstance(scheme, dict):
                return False
            for scheme_type, _ in scheme.items():
                if scheme_type not in security_definitions:
                    return False
        return True


check = SecurityRequirement()



from __future__ import annotations

from abc import abstractmethod
from collections.abc import Iterable
from typing import Dict, List, Any

from checkov.common.checks.base_check import BaseCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.data.registry import data_registry


class BaseDataCheck(BaseCheck):
    def __init__(
        self,
        name: str,
        id: str,
        categories: Iterable[CheckCategories],
        supported_data: Iterable[str],
        guideline: str | None = None,
    ) -> None:
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_data,
                         block_type="data", guideline=guideline)
        self.supported_data = supported_data
        data_registry.register(self)

    def scan_entity_conf(self, conf: Dict[str, List[Any]], entity_type: str) -> CheckResult:
        self.entity_type = entity_type

        if conf.get("count") == [0]:
            return CheckResult.UNKNOWN

        return self.scan_data_conf(conf)

    @abstractmethod
    def scan_data_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        raise NotImplementedError()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck
from typing import List


class SQLServerThreatDetectionTypes(BaseResourceCheck):
    def __init__(self):
        name = "Ensure that 'Threat Detection types' is set to 'All'"
        id = "CKV_AZURE_25"
        supported_resources = ['azurerm_mssql_server_security_alert_policy']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'disabled_alerts' in conf and any(conf['disabled_alerts'][0]):
            return CheckResult.FAILED
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ['disabled_alerts']


check = SQLServerThreatDetectionTypes()



from typing import Dict, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck


# https://docs.microsoft.com/en-us/azure/templates/microsoft.sql/2019-06-01-preview/servers
# https://docs.microsoft.com/en-us/azure/templates/microsoft.sql/2017-03-01-preview/servers/securityalertpolicies


class SQLServerThreatDetectionTypes(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that 'Threat Detection types' is set to 'All'"
        id = "CKV_AZURE_25"
        supported_resources = ("Microsoft.Sql/servers/databases",)  # 'Microsoft.Sql/servers'
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        resources = conf.get("resources")
        if isinstance(resources, list):
            for resource in resources:
                if "type" in resource:
                    if resource["type"] in (
                        "Microsoft.Sql/servers/databases/securityAlertPolicies",
                        "securityAlertPolicies",
                    ):
                        properties = resource.get("properties")
                        if isinstance(properties, dict):
                            if "state" in properties and properties["state"].lower() == "enabled":
                                if not properties.get("disabledAlerts"):
                                    return CheckResult.PASSED

        return CheckResult.FAILED


check = SQLServerThreatDetectionTypes()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class IAMPasswordLength(BaseResourceCheck):
    def __init__(self) -> None:
        name = "OCI IAM password policy for local (non-federated) users has a minimum length of 14 characters"
        id = "CKV_OCI_18"
        supported_resources = ('oci_identity_authentication_policy',)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if 'password_policy' in conf.keys():
            self.evaluated_keys = ["password_policy"]
            rules = conf.get("password_policy")[0]
            if 'minimum_password_length' in rules:
                password_length = rules.get("minimum_password_length")
                if isinstance(password_length[0], int) and password_length[0] < 14:
                    self.evaluated_keys = ["password_policy/minimum_password_length"]
                    return CheckResult.FAILED
                return CheckResult.PASSED
            return CheckResult.FAILED
        return CheckResult.FAILED


check = IAMPasswordLength()



from __future__ import annotations

import json
from collections import defaultdict
from operator import itemgetter

from checkov.common.models.enums import CheckResult
from typing import Any, TYPE_CHECKING

if TYPE_CHECKING:
    from checkov.common.output.record import Record
    from checkov.common.output.report import Report
    from checkov.common.typing import _BaselineFinding, _BaselineFailedChecks


class Baseline:
    def __init__(self, output_skipped: bool = False) -> None:
        self.path = ""
        self.path_failed_checks_map: dict[str, list[_BaselineFinding]] = defaultdict(list)
        self.failed_checks: list[_BaselineFailedChecks] = []
        self.output_skipped = output_skipped

    def add_findings_from_report(self, report: Report) -> None:
        for check in report.failed_checks:
            try:
                existing = next(
                    x for x in self.path_failed_checks_map[check.file_path] if x["resource"] == check.resource  # type:ignore[has-type]
                )
            except StopIteration:
                existing = {"resource": check.resource, "check_ids": []}  # type:ignore[has-type]
                self.path_failed_checks_map[check.file_path].append(existing)
            existing["check_ids"].append(check.check_id)
            existing["check_ids"].sort()  # Sort the check IDs to be nicer to the eye

    def to_dict(self) -> dict[str, Any]:
        """
        The output of this class needs to be very explicit, hence the following structure of the dict:
        {
            "failed_checks": [
                {
                    "file": "path/to/file",
                    "findings: [
                        {
                            "resource": "aws_s3_bucket.this",
                            "check_ids": [
                                "CKV_AWS_1",
                                "CKV_AWS_2",
                                "CKV_AWS_3"
                            ]
                        }
                    ]
                }
            ]
        }
        """
        failed_checks_list = []
        for file, findings in self.path_failed_checks_map.items():
            formatted_findings = []
            for finding in findings:
                formatted_findings.append({"resource": finding["resource"], "check_ids": finding["check_ids"]})
            sorted_findings = sorted(formatted_findings, key=itemgetter("resource"))
            failed_checks_list.append({"file": file, "findings": sorted_findings})

        resp = {"failed_checks": sorted(failed_checks_list, key=itemgetter("file"))}
        return resp

    def compare_and_reduce_reports(self, scan_reports: list[Report]) -> None:
        for scan_report in scan_reports:
            scan_report.passed_checks = [
                check for check in scan_report.passed_checks if self._is_check_in_baseline(check)
            ]
            scan_report.skipped_checks = [
                check for check in scan_report.skipped_checks if self._is_check_in_baseline(check)
            ]
            if self.output_skipped:
                for check in scan_report.failed_checks:
                    if self._is_check_in_baseline(check):
                        check.check_result["suppress_comment"] = "baseline-skipped"
                        check.check_result["result"] = CheckResult.SKIPPED
                        scan_report.skipped_checks.append(check)
            scan_report.failed_checks = [
                check for check in scan_report.failed_checks if not self._is_check_in_baseline(check)
            ]

    def _is_check_in_baseline(self, check: Record) -> bool:
        failed_check_id = check.check_id
        failed_check_resource = check.resource  # type:ignore[has-type]
        for baseline_failed_check in self.failed_checks:
            for finding in baseline_failed_check["findings"]:
                if finding["resource"] == failed_check_resource and failed_check_id in finding["check_ids"]:
                    return True
        return False

    def from_json(self, file_path: str) -> None:
        self.path = file_path
        with open(file_path, "r") as f:
            baseline_raw = json.load(f)
            self.failed_checks = baseline_raw.get("failed_checks", {})



import logging
import os
from io import StringIO

from checkov.common.resource_code_logger_filter import add_resource_code_filter_to_logger

LOG_LEVEL = os.getenv('LOG_LEVEL', 'WARNING').upper()
logging.basicConfig(level=LOG_LEVEL)
log_formatter = logging.Formatter("%(asctime)s [%(threadName)-12.12s] [%(levelname)-5.5s]  %(message)s")
root_logger = logging.getLogger()
add_resource_code_filter_to_logger(root_logger)
stream_handler = root_logger.handlers[0]
stream_handler.setFormatter(log_formatter)
stream_handler.setLevel(LOG_LEVEL)
root_logger.setLevel(logging.DEBUG)
logging.getLogger().setLevel(logging.DEBUG)
logging.getLogger("urllib3").setLevel(logging.ERROR)
logging.getLogger("urllib3.connectionpool").setLevel(logging.ERROR)
logging.getLogger("urllib3.connectionpool").propagate = False
logging.getLogger("urllib3").propagate = False
log_stream = StringIO()
stream_handler = logging.StreamHandler(stream=log_stream)
stream_handler.setFormatter(log_formatter)
stream_handler.setLevel(logging.DEBUG)
root_logger.addHandler(stream_handler)



from __future__ import annotations

from typing import Any
from abc import abstractmethod

from bc_jsonpath_ng import parse

from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.github.base_github_configuration_check import BaseGithubCheck
from checkov.github.schemas.branch_protection import schema as branch_security_schema
from checkov.github.schemas.no_branch_protection import schema as no_branch_security_schema
from checkov.json_doc.enums import BlockType

MESSAGE_BRANCH_NOT_PROTECTED = "Branch not protected"


class NegativeBranchSecurity(BaseGithubCheck):
    def __init__(self, id: str, name: str, missing_attribute_result: CheckResult = CheckResult.PASSED) -> None:
        categories = (CheckCategories.SUPPLY_CHAIN,)
        super().__init__(
            id=id,
            name=name,
            categories=categories,
            supported_entities=("*",),
            block_type=BlockType.DOCUMENT,
        )
        self.missing_attribute_result = missing_attribute_result

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> CheckResult:
        if branch_security_schema.validate(conf):
            evaluated_key = self.get_evaluated_keys()[0].replace("/", ".")
            jsonpath_expression = parse(f"$..{evaluated_key}")
            matches = jsonpath_expression.find(conf)
            if not matches:
                # attribute doesn't exists
                return self.missing_attribute_result

            if matches:
                forbidden_values = self.get_forbidden_values()
                if ANY_VALUE in forbidden_values or any(
                    match.value in forbidden_values for match in matches
                ):
                    # attribute exists, but is not a value of 'get_forbidden_values()' or 'ANY_VALUE'
                    return CheckResult.FAILED

            return CheckResult.PASSED
        if no_branch_security_schema.validate(conf):
            message = conf.get("message", "")
            if message == MESSAGE_BRANCH_NOT_PROTECTED:
                return CheckResult.FAILED
        return CheckResult.UNKNOWN

    @abstractmethod
    def get_evaluated_keys(self) -> list[str]:
        """List of JSONPath syntax path of the checked attributes"""
        pass

    @abstractmethod
    def get_forbidden_values(self) -> list[Any]:
        """List of forbidden values for the evaluated keys"""
        pass



from __future__ import annotations

import concurrent.futures
import logging
import multiprocessing
import os
import platform
from collections.abc import Iterator, Iterable
from multiprocessing.pool import Pool
from typing import Any, List, Generator, Callable, Optional, TypeVar, TYPE_CHECKING

from checkov.common.models.enums import ParallelizationType

if TYPE_CHECKING:
    from multiprocessing.connection import Connection

_T = TypeVar("_T")


class ParallelRunner:
    def __init__(
        self, workers_number: int | None = None, parallelization_type: ParallelizationType = ParallelizationType.FORK
    ) -> None:
        self.workers_number = (workers_number if workers_number else os.cpu_count()) or 1
        self.os = platform.system()
        self.type: str | ParallelizationType = parallelization_type

        custom_type = os.getenv("CHECKOV_PARALLELIZATION_TYPE")
        if custom_type:
            self.type = custom_type

        if not custom_type and os.getenv("PYCHARM_HOSTED") == "1":
            # PYCHARM_HOSTED env variable equals 1 when debugging via jetbrains IDE.
            # To prevent JetBrains IDE from crashing on debug run sequentially
            self.type = ParallelizationType.NONE
        elif self.os == "Windows":
            # 'fork' mode is not supported on 'Windows'
            # 'spawn' mode results in a strange error, which needs to be investigated on an actual Windows machine
            self.type = ParallelizationType.THREAD

    def run_function(
        self,
        func: Callable[..., _T],
        items: List[Any],
        group_size: Optional[int] = None,
    ) -> Iterable[_T]:
        if self.type == ParallelizationType.THREAD:
            return self._run_function_multithreaded(func, items)
        elif self.type == ParallelizationType.FORK:
            return self._run_function_multiprocess_fork(func, items, group_size)
        elif self.type == ParallelizationType.SPAWN:
            return self._run_function_multiprocess_spawn(func, items, group_size)
        else:
            return self._run_function_sequential(func, items)

    def _run_function_multiprocess_fork(
        self, func: Callable[[Any], _T], items: List[Any], group_size: Optional[int]
    ) -> Generator[_T, None, None]:
        if not group_size:
            group_size = int(len(items) / self.workers_number) + 1
        groups_of_items = [items[i : i + group_size] for i in range(0, len(items), group_size)]

        def func_wrapper(original_func: Callable[[Any], _T], items_group: List[Any], connection: Connection) -> None:
            for item in items_group:
                try:
                    if isinstance(item, tuple):
                        # unpack a tuple to pass multiple arguments to the target function
                        result = original_func(*item)
                    else:
                        result = original_func(item)
                except Exception:
                    logging.error(
                        f"Failed to invoke function {func.__code__.co_filename.replace('.py', '')}.{func.__name__} with {item}",
                        exc_info=True,
                    )
                    result = None

                connection.send(result)
            connection.close()

        logging.debug(
            f"Running function {func.__code__.co_filename.replace('.py', '')}.{func.__name__} with parallelization type 'fork'"
        )
        processes = []
        for group_of_items in groups_of_items:
            parent_conn, child_conn = multiprocessing.Pipe(duplex=False)
            process = multiprocessing.get_context("fork").Process(
                target=func_wrapper, args=(func, group_of_items, child_conn)
            )
            processes.append((process, parent_conn, len(group_of_items)))
            process.start()

        for _, parent_conn, group_len in processes:
            for _ in range(group_len):
                try:
                    yield parent_conn.recv()
                except EOFError:
                    pass

    def _run_function_multiprocess_spawn(
        self, func: Callable[[Any], _T], items: list[Any], group_size: int | None
    ) -> Iterable[_T]:
        if multiprocessing.current_process().daemon:
            # can't create a new pool, when already inside a pool
            return self._run_function_multithreaded(func, items)

        if not group_size:
            group_size = int(len(items) / self.workers_number) + 1

        logging.debug(
            f"Running function {func.__code__.co_filename.replace('.py', '')}.{func.__name__} with parallelization type 'spawn'"
        )
        with Pool(processes=self.workers_number, context=multiprocessing.get_context("spawn")) as p:
            if items and isinstance(items[0], tuple):
                # need to use 'starmap' to pass multiple arguments to the target function
                return p.starmap(func, items, chunksize=group_size)

            return p.map(func, items, chunksize=group_size)

    def _run_function_multithreaded(self, func: Callable[[Any], _T], items: List[Any]) -> Iterator[_T]:
        logging.debug(
            f"Running function {func.__code__.co_filename.replace('.py', '')}.{func.__name__} with parallelization type 'thread'"
        )
        with concurrent.futures.ThreadPoolExecutor(max_workers=self.workers_number) as executor:
            if items and isinstance(items[0], tuple):
                # split a list of tuple into tuples of the positioned values of the tuple
                return executor.map(func, *list(zip(*items)))  # noqa[B905]  # no need to set 'strict' otherwise 'mypy' complains

            return executor.map(func, items)

    def _run_function_sequential(self, func: Callable[[Any], _T], items: List[Any]) -> Iterator[_T]:
        logging.debug(
            f"Running function {func.__code__.co_filename.replace('.py', '')}.{func.__name__} with parallelization type 'none'"
        )
        if items and isinstance(items[0], tuple):
            # unpack a tuple to pass multiple arguments to the target function
            return (func(*item) for item in items)

        return (func(item) for item in items)


parallel_runner = ParallelRunner()



from __future__ import annotations

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_check import BaseResourceCheck

from typing import Any

from checkov.common.util.data_structures_utils import find_in_dict


class VMEncryptionAtHostEnabled(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that Virtual machine scale sets have encryption at host enabled"
        id = "CKV_AZURE_97"
        supported_resources = ("Microsoft.Compute/virtualMachineScaleSets", "Microsoft.Compute/virtualMachines")
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        encryption = ""

        if self.entity_type == "Microsoft.Compute/virtualMachines":
            self.evaluated_keys = ["properties/securityProfile/encryptionAtHost"]
            encryption = find_in_dict(input_dict=conf, key_path="properties/securityProfile/encryptionAtHost")
        elif self.entity_type == "Microsoft.Compute/virtualMachineScaleSets":
            self.evaluated_keys = ["properties/virtualMachineProfile/securityProfile/encryptionAtHost"]
            encryption = find_in_dict(
                input_dict=conf, key_path="properties/virtualMachineProfile/securityProfile/encryptionAtHost"
            )

        if str(encryption).lower() == "true":
            return CheckResult.PASSED

        return CheckResult.FAILED


check = VMEncryptionAtHostEnabled()



from __future__ import annotations

from abc import abstractmethod
from collections.abc import Iterable
from typing import Any, TYPE_CHECKING

from checkov.common.checks.base_check import BaseCheck
from checkov.serverless.checks.custom.registry import custom_registry

if TYPE_CHECKING:
    from checkov.common.models.enums import CheckCategories, CheckResult


class BaseCustomCheck(BaseCheck):
    def __init__(
        self,
        name: str,
        id: str,
        categories: Iterable[CheckCategories],
        supported_entities: Iterable[str],
        guideline: str | None = None,
    ) -> None:
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_entities,
            block_type="serverless",
            guideline=guideline,
        )
        custom_registry.register(self)

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> CheckResult:
        return self.scan_custom_conf(conf)

    @abstractmethod
    def scan_custom_conf(self, conf: dict[str, Any]) -> CheckResult:
        raise NotImplementedError()



from __future__ import annotations

from typing import Any
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.checks.enums import BlockType
from checkov.openapi.checks.resource.v2.BaseOpenapiCheckV2 import BaseOpenapiCheckV2


class Oauth2SecurityDefinitionImplicitFlow(BaseOpenapiCheckV2):
    def __init__(self) -> None:
        id = "CKV_OPENAPI_12"
        name = "Ensure no security definition is using implicit flow on OAuth2, which is deprecated - version 2.0 files"
        categories = [CheckCategories.API_SECURITY]
        supported_resources = ["securityDefinitions"]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_resources,
            block_type=BlockType.DOCUMENT,
        )

    def scan_openapi_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]]:
        security_definitions = conf.get('securityDefinitions') or {}

        for auth_key, auth_dict in security_definitions.items():
            if self.is_start_end_line(auth_key):
                continue
            if not isinstance(auth_dict, dict):
                return CheckResult.UNKNOWN, conf
            auth_type = auth_dict.get('type', '')
            if auth_type.lower() == 'oauth2':
                auth_flow = auth_dict.get('flow', '')
                if auth_flow == 'implicit':
                    return CheckResult.FAILED, auth_dict

        return CheckResult.PASSED, conf


check = Oauth2SecurityDefinitionImplicitFlow()



from __future__ import annotations

from checkov.common.bridgecrew.check_type import CheckType
from checkov.sast.checks_infra.base_registry import Registry


class BaseCdkRegistry(Registry):
    def __init__(self, checks_dir: str) -> None:
        super().__init__(checks_dir=checks_dir)
        self.report_type = CheckType.CDK



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class APIGatewayAuthorization(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure there is no open access to back-end resources through API"
        id = "CKV_AWS_59"
        supported_resources = ('AWS::ApiGateway::Method',)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        if 'Properties' in conf.keys():
            if 'HttpMethod' in conf['Properties'].keys() and 'AuthorizationType' in conf['Properties'].keys():
                if conf['Properties']['HttpMethod'] != "OPTIONS" and conf['Properties']['AuthorizationType'] == "NONE":
                    if 'ApiKeyRequired' not in conf['Properties'].keys() or conf['Properties']['ApiKeyRequired'] is False:
                        return CheckResult.FAILED
        return CheckResult.PASSED


check = APIGatewayAuthorization()



from typing import Optional, List
from checkov.common.bridgecrew.severities import Severity
from checkov.common.models.enums import CheckCategories


class BaseSastCheck:
    def __init__(self, name: str, id: str, severity: Optional[Severity] = None) -> None:
        self.name: str = name
        self.id: str = id
        self.categories: List[CheckCategories] = [CheckCategories.SAST]
        # TODO
        self.guideline: str = ''
        self.severity: Optional[Severity] = severity
        self.bc_id: str = ''



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.util.data_structures_utils import find_in_dict


class AzureManagedDiscEncryption(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Azure managed disk have encryption enabled"
        id = "CKV_AZURE_2"
        supported_resources = ("Microsoft.Compute/disks",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        properties = conf.get("properties")
        if properties:
            encryption = properties.get("encryption")
            if encryption:
                # if the block exists, then it is enabled
                return CheckResult.PASSED

            encryption_enabled = find_in_dict(input_dict=properties, key_path="encryptionSettingsCollection/enabled")
            if str(encryption_enabled).lower() == "true":
                return CheckResult.PASSED

            encryption_enabled = find_in_dict(input_dict=properties, key_path="encryptionSettings/enabled")
            if str(encryption_enabled).lower() == "true":
                return CheckResult.PASSED

        return CheckResult.FAILED


check = AzureManagedDiscEncryption()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_check import BaseResourceCheck


class VMDisablePasswordAuthentication(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that Virtual machine does not enable password authentication"
        id = "CKV_AZURE_149"
        supported_resources = (
            "Microsoft.Compute/virtualMachineScaleSets",
            "Microsoft.Compute/virtualMachines",
        )
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        os_profile = None

        properties = conf.get("properties")
        if properties and isinstance(properties, dict):
            if self.entity_type == "Microsoft.Compute/virtualMachines":
                tmp_os_profile = properties.get("osProfile")
                if tmp_os_profile and isinstance(tmp_os_profile, dict):
                    os_profile = tmp_os_profile
            elif self.entity_type == "Microsoft.Compute/virtualMachineScaleSets":
                vm_profile = properties.get("virtualMachineProfile")
                if vm_profile and isinstance(vm_profile, dict):
                    tmp_os_profile = vm_profile.get("osProfile")
                    if tmp_os_profile and isinstance(tmp_os_profile, dict):
                        os_profile = tmp_os_profile

            if os_profile is None:
                return CheckResult.UNKNOWN

            linux_config = os_profile.get("linuxConfiguration")
            if linux_config and isinstance(linux_config, dict):
                pass_auth = linux_config.get("disablePasswordAuthentication")
                if pass_auth and isinstance(pass_auth, bool):
                    return CheckResult.PASSED if pass_auth and isinstance(pass_auth, bool) else CheckResult.FAILED
                return CheckResult.FAILED

            return CheckResult.UNKNOWN

        return CheckResult.FAILED


check = VMDisablePasswordAuthentication()



from checkov.common.checks_infra.solvers.attribute_solvers.any_attribute_solver import AnyResourceSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.contains_attribute_solver import ContainsAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.not_contains_attribute_solver import NotContainsAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.ending_with_attribute_solver import EndingWithAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.equals_attribute_solver import EqualsAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.regex_match_attribute_solver import RegexMatchAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.exists_attribute_solver import ExistsAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.not_ending_with_attribute_solver import NotEndingWithAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.not_equals_attribute_solver import NotEqualsAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.not_regex_match_attribute_solver import NotRegexMatchAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.not_exists_attribute_solver import NotExistsAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.not_starting_with_attribute_solver import NotStartingWithAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.starting_with_attribute_solver import StartingWithAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.within_attribute_solver import WithinAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.not_within_attribute_solver import NotWithinAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.greater_than_attribute_solver import GreaterThanAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.greater_than_or_equal_attribute_solver import GreaterThanOrEqualAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.less_than_attribute_solver import LessThanAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.less_than_or_equal_attribute_solver import LessThanOrEqualAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.subset_attribute_solver import SubsetAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.not_subset_attribute_solver import NotSubsetAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.is_empty_attribute_solver import IsEmptyAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.is_not_empty_attribute_solver import IsNotEmptyAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.length_equals_attribute_solver import LengthEqualsAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.length_not_equals_attribute_solver import LengthNotEqualsAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.length_greater_than_attribute_solver import LengthGreaterThanAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.length_greater_than_or_equal_attribute_solver import LengthGreaterThanOrEqualAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.length_less_than_attribute_solver import LengthLessThanAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.length_less_than_or_equal_attribute_solver import LengthLessThanOrEqualAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.is_true_attribute_solver import IsTrueAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.is_false_attribute_solver import IsFalseAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.intersects_attribute_solver import IntersectsAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.not_intersects_attribute_solver import NotIntersectsAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.equals_ignore_case_attribute_solver import EqualsIgnoreCaseAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.not_equals_ignore_case_attribute_solver import NotEqualsIgnoreCaseAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.range_includes_attribute_solver import RangeIncludesAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.range_not_includes_attribute_solver import RangeNotIncludesAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.number_of_words_equals_attribute_solver import NumberOfWordsEqualsAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.number_of_words_not_equals_attribute_solver import NumberOfWordsNotEqualsAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.number_of_words_greater_than_attribute_solver import NumberOfWordsGreaterThanAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.number_of_words_greater_than_or_equal_attribute_solver import NumberOfWordsGreaterThanOrEqualAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.number_of_words_less_than_or_equal_attribute_solver import NumberOfWordsLessThanOrEqualAttributeSolver  # noqa
from checkov.common.checks_infra.solvers.attribute_solvers.number_of_words_less_than_attribute_solver import NumberOfWordsLessThanAttributeSolver  # noqa



from __future__ import annotations

import logging
import time
from pathlib import Path
from typing import Any

from requests import JSONDecodeError

from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.util.http_utils import request_wrapper

from checkov.common.util.tqdm_utils import ProgressBar

SLEEP_DURATION = 5
MAX_SLEEP_DURATION = 240


class Scanner:
    def __init__(self, pbar: ProgressBar | None = None, root_folder: str | Path | None = None) -> None:
        self._base_url = bc_integration.api_url
        self.bc_cli_scan_api_url = f"{self._base_url}/api/v1/vulnerabilities/cli/scan"
        if pbar:
            self.pbar = pbar
        else:
            self.pbar = ProgressBar('')
            self.pbar.turn_off_progress_bar()
        self.root_folder = root_folder

    def scan(self) -> dict[str, Any] | None:
        """run SCA package scan and poll scan results"""
        if not self.run_scan():
            return None
        return self.poll_scan_result()

    def run_scan(self) -> bool:
        try:
            logging.info("Start to scan package files.")

            request_body = {
                "branch": "",
                "commit": "",
                "path": bc_integration.repo_path,
                "repoId": bc_integration.repo_id,
                "id": bc_integration.timestamp,
                "repositoryId": ""
            }

            response = request_wrapper(
                "POST", self.bc_cli_scan_api_url,
                headers=bc_integration.get_default_headers("POST"),
                json=request_body,
                should_call_raise_for_status=True
            )

            response_json = response.json()

            if not response_json["startedSuccessfully"]:
                logging.info("Failed to run package scanning.")
                return False
            return True
        except Exception:
            logging.debug(
                "[sca_package_2] - Unexpected failure happened during package scanning.\n"
                "the scanning is terminating. details are below.\n"
                "please try again. if it is repeated, please report.", exc_info=True)
            return False

    def poll_scan_result(self) -> dict[str, Any]:
        total_sleeping_time = 0

        while total_sleeping_time < MAX_SLEEP_DURATION:
            response = request_wrapper(
                "GET", f"{self.bc_cli_scan_api_url}/{bc_integration.timestamp}",
                headers=bc_integration.get_default_headers("GET"),
                params={"repoId": bc_integration.repo_id}
            )

            try:
                response_json = response.json()
            except JSONDecodeError:
                logging.debug(f"Unexpected response from {self.bc_cli_scan_api_url}: {response.text}")
                return {}

            current_state = response_json.get("status", "")
            if not current_state:
                logging.error("Failed to poll scan results.")
                return {}

            if current_state == "COMPLETED":
                logging.debug(response_json)
                report_url = response_json['reportUrl']
                report_response = request_wrapper("GET", report_url, headers={'Accept': 'application/json'})
                return report_response.json()  # type: ignore

            if current_state == "FAILED":
                logging.debug(response_json)
                return {}

            time.sleep(SLEEP_DURATION)
            total_sleeping_time += SLEEP_DURATION

        logging.debug(f"Timeout, slept for {total_sleeping_time}")
        return {}



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import \
    BaseResourceCheck


class CloudWatchLogGroupRetentionYear(BaseResourceCheck):
    def __init__(self) -> None:
        """
        NIST.800-53.r5 AU-10, NIST.800-53.r5 AU-11, NIST.800-53.r5 AU-6(3), NIST.800-53.r5 AU-6(4),
        NIST.800-53.r5 CA-7, NIST.800-53.r5 SI-12
        CloudWatch log groups should be retained for at least 1 year
        """
        name = "Ensure CloudWatch log groups retains logs for at least 1 year"
        id = "CKV_AWS_338"
        supported_resource = ("aws_cloudwatch_log_group",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resource)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        retention = conf.get("retention_in_days")
        if retention and isinstance(retention, list):
            retention = retention[0]
            if not isinstance(retention, int):
                # probably a dependent variable
                return CheckResult.UNKNOWN
            # If you select 0, the events in the log group are always retained and never expire.
            if retention == 0 or retention >= 365:
                return CheckResult.PASSED

        return CheckResult.FAILED


check = CloudWatchLogGroupRetentionYear()



import logging
from typing import Any, Dict, Union, List, Optional

from checkov.common.output.report import Report
from checkov.common.sast.consts import POLICIES_ERRORS, POLICIES_ERRORS_COUNT, SOURCE_FILES_COUNT, POLICY_COUNT, SastLanguages


class SastReport(Report):
    def __init__(self, check_type: str, metadata: Dict[str, Optional[Union[str, int, List[str]]]], language: SastLanguages):
        super().__init__(check_type)
        self.metadata = metadata
        self.language: SastLanguages = language
        self.sast_imports: Dict[str, Any] = {}
        self.sast_reachability: Dict[str, Any] = {}

    def get_summary(self) -> Dict[str, Union[int, str]]:
        base_summary: Dict[str, Union[int, str]] = super().get_summary()

        err_str = ""
        policies_errors_count = 0
        policies_errors = self.metadata.get(POLICIES_ERRORS)
        if isinstance(policies_errors, list) and policies_errors:
            policies_errors_count = len(policies_errors)
            for e in policies_errors:
                err_str += f"\t- {e}\n"
        base_summary[POLICIES_ERRORS] = err_str
        base_summary[POLICIES_ERRORS_COUNT] = policies_errors_count
        source_files_count = self.metadata.get(SOURCE_FILES_COUNT)
        if isinstance(source_files_count, int) or isinstance(source_files_count, str):
            base_summary[SOURCE_FILES_COUNT] = str(source_files_count)

        policy_count = self.metadata.get(POLICY_COUNT)
        if isinstance(policy_count, int) or isinstance(policy_count, str):
            base_summary[POLICY_COUNT] = policy_count

        return base_summary

    @staticmethod
    def get_formated_reachability_report(reachability_report_dict: Dict[SastLanguages, Any]) -> Dict[str, Any]:
        formated_report: Dict[str, Any] = {}
        for lang, repos_data in reachability_report_dict.items():
            formated_report[lang.value] = []
            for repo_name, files_data in repos_data.items():
                new_repo = {'Name': repo_name, 'Files': []}
                for file_path, packages_data in files_data['files'].items():
                    new_file = {'Path': file_path, 'Packages': []}
                    for package_name, package_data in packages_data['packages'].items():
                        new_package = {'Name': package_name, 'Alias': package_data['alias'], 'Functions': []}
                        for func in package_data['functions']:
                            new_func = {'Name': func['name'], 'Alias': func['alias'], 'LineNumber': func['line_number'], 'CodeBlock': [func['code_block']], 'CveId': func.get('cve_id', '')}
                            new_package['Functions'].append(new_func)
                        new_file['Packages'].append(new_package)
                    new_repo['Files'].append(new_file)
                formated_report[lang.value].append(new_repo)
        return formated_report


class SastData:
    def __init__(self) -> None:
        self.imports_data: Optional[Dict[str, Any]] = None
        self.reachability_report: Optional[Dict[str, Any]] = None

    def set_imports_data(self, imports_data: Dict[str, Any]) -> None:
        self.imports_data = imports_data

    def set_reachability_report(self, reachability_report: Dict[str, Any]) -> None:
        self.reachability_report = reachability_report

    @staticmethod
    def get_sast_import_report(scan_reports: List[SastReport]) -> Dict[str, Any]:
        sast_imports_report: Dict[SastLanguages, Any] = {}
        for report in scan_reports:
            sast_imports_report[report.language] = {}
        for report in scan_reports:
            for file_name, all_data in report.sast_imports.items():
                current_imports = all_data.get('all', [])
                if current_imports:
                    sast_imports_report[report.language][file_name] = {'all': current_imports}
                    aliases = all_data.get('aliases', {})
                    sast_imports_report[report.language][file_name]['aliases'] = aliases
        return {"imports": sast_imports_report}

    @staticmethod
    def get_sast_reachability_report(scan_reports: List[SastReport]) -> Dict[str, Any]:
        first_found_repo_name = None
        sast_reachability_report: Dict[SastLanguages, Any] = {}
        for report in scan_reports:
            sast_reachability_report[report.language] = {}
        for report in scan_reports:
            for repo_name, repo_data in report.sast_reachability.items():

                # validating we are dealing only with one repo, as it happens for imports report
                if first_found_repo_name:
                    if repo_name != first_found_repo_name:
                        logging.error(f'[get_sast_reachability_report] - found more than one repository in '
                                      f'the scan reports. {scan_reports}')
                        return {"reachability": {}}
                else:
                    first_found_repo_name = repo_name

                for file_name, file_data in repo_data.files.items():
                    sast_reachability_report[report.language][file_name] = file_data
        return {"reachability": sast_reachability_report}



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories
from typing import Any, List


class APIGatewayV2RouteDefinesAuthorizationType(BaseResourceValueCheck):

    def __init__(self):
        """
        NIST.800-53.r5 AC-3, NIST.800-53.r5 CM-2, NIST.800-53.r5 CM-2(2)
        """
        name = "Ensure API GatewayV2 routes specify an authorization type"
        id = "CKV_AWS_309"
        supported_resources = ['aws_apigatewayv2_route']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "authorization_type"

    def get_expected_values(self) -> List[Any]:
        return ["AWS_IAM", "CUSTOM", "JWT"]


check = APIGatewayV2RouteDefinesAuthorizationType()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck
import re

SUBSCRIPTION = re.compile(r"\/|\/subscriptions\/[\w\d-]+$|\[subscription\(\).id\]")

# https://docs.microsoft.com/en-us/azure/role-based-access-control/custom-roles-template
# https://docs.microsoft.com/en-us/azure/role-based-access-control/role-definitions
# https://docs.microsoft.com/en-us/azure/templates/microsoft.authorization/2018-01-01-preview/roledefinitions


class CustomRoleDefinitionSubscriptionOwner(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that no custom subscription owner roles are created"
        id = "CKV_AZURE_39"
        supported_resources = ("Microsoft.Authorization/roleDefinitions",)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        if "properties" in conf:
            if "assignableScopes" in conf["properties"]:
                if any(
                    isinstance(scope, str) and re.match(SUBSCRIPTION, scope)
                    for scope in conf["properties"]["assignableScopes"]
                ):
                    if "permissions" in conf["properties"]:
                        if conf["properties"]["permissions"]:
                            for permission in conf["properties"]["permissions"]:
                                if "actions" in permission and "*" in permission["actions"]:
                                    return CheckResult.FAILED
        return CheckResult.PASSED


check = CustomRoleDefinitionSubscriptionOwner()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class RedisCacheStandardReplicationEnabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        """
        With Standard Replication, Azure Cache for Redis has a high availability architecture
        that ensures your managed instance is functioning, even when outages affect
        the underlying virtual machines (VMs). Whether the outage is planned or unplanned outages,
        Azure Cache for Redis delivers greater percentage availability rates than what's attainable
        by hosting Redis on a single VM.

        An Azure Cache for Redis in the applicable tiers runs on a pair of Redis servers by default.
        The two servers are hosted on dedicated VMs.
        Open-source Redis allows only one server to handle data write requests.
        """
        name = "Standard Replication should be enabled"
        id = "CKV_AZURE_230"
        supported_resources = ("azurerm_redis_cache",)
        categories = (CheckCategories.BACKUP_AND_RECOVERY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "sku_name"

    def get_expected_values(self) -> list[Any]:
        return ["Standard", "Premium"]


check = RedisCacheStandardReplicationEnabled()



from __future__ import annotations

import re
from abc import abstractmethod
from collections.abc import Iterable
from typing import Any

from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.models.consts import ANY_VALUE
from checkov.common.util.data_structures_utils import find_in_dict
from checkov.common.util.type_forcers import force_list

VARIABLE_DEPENDANT_REGEX = re.compile(r"(?:parameters|variables)\(")


class BaseResourceNegativeValueCheck(BaseResourceCheck):
    def __init__(
        self,
        name: str,
        id: str,
        categories: "Iterable[CheckCategories]",
        supported_resources: "Iterable[str]",
        missing_block_result: CheckResult = CheckResult.PASSED,
        guideline: str | None = None,
    ) -> None:
        super().__init__(
            name=name, id=id, categories=categories, supported_resources=supported_resources, guideline=guideline
        )
        self.missing_block_result = missing_block_result

    @staticmethod
    def _is_variable_dependant(value: Any) -> bool:
        return bool(isinstance(value, str) and re.match(VARIABLE_DEPENDANT_REGEX, value))

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        inspected_key = self.get_inspected_key()
        forbidden_values = self.get_forbidden_values()
        value = find_in_dict(conf, inspected_key)
        if value:
            if isinstance(value, list) and len(value) == 1:
                value = value[0]

            if self._is_variable_dependant(value):
                # If the tested attribute is variable-dependant, then result is PASSED
                return CheckResult.UNKNOWN

            if value in forbidden_values or ANY_VALUE in forbidden_values:
                return CheckResult.FAILED
            else:
                return CheckResult.PASSED

        return self.missing_block_result

    @abstractmethod
    def get_inspected_key(self) -> str:
        """
        :return: JSONPath syntax path of the checked attribute
        """
        raise NotImplementedError()

    @abstractmethod
    def get_forbidden_values(self) -> list[Any]:
        """
        Returns a list of vulnerable values for the inspected key, governed by provider best practices
        """
        raise NotImplementedError()

    def get_evaluated_keys(self) -> list[str]:
        return force_list(self.get_inspected_key())



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck


class SQLServerNoPublicAccess(BaseResourceCheck):
    def __init__(self) -> None:
        # https://docs.microsoft.com/en-us/azure/templates/microsoft.sql/2019-06-01-preview/servers
        # https://docs.microsoft.com/en-us/azure/templates/microsoft.sql/2014-04-01/servers/firewallrules
        name = "Ensure no SQL Databases allow ingress from 0.0.0.0/0 (ANY IP)"
        id = "CKV_AZURE_11"
        supported_resources = ("Microsoft.Sql/servers",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

        # API Version 2015-05-01-preview and 2014-04-01

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        resources = conf.get("resources")
        if resources and isinstance(resources, list):
            for resource in resources:
                resource_type = resource.get("type")
                if resource_type in ("Microsoft.Sql/servers/firewallRules", "firewallRules", "firewallrules"):
                    if "properties" in resource:
                        if (
                            "startIpAddress" in resource["properties"]
                            and resource["properties"]["startIpAddress"] in ["0.0.0.0", "0.0.0.0/0"]  # nosec  # false positive
                            and "endIpAddress" in resource["properties"]
                            and resource["properties"]["endIpAddress"] == "255.255.255.255"
                        ):
                            return CheckResult.FAILED
        return CheckResult.PASSED


check = SQLServerNoPublicAccess()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class MSSQLServerAuditPolicyLogMonitor(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure default Auditing policy for a SQL Server is configured to capture and retain the activity logs"
        id = "CKV_AZURE_156"
        supported_resources = ['azurerm_mssql_database_extended_auditing_policy']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.FAILED)

    def get_inspected_key(self):
        return "log_monitoring_enabled"


check = MSSQLServerAuditPolicyLogMonitor()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class ACRGeoreplicated(BaseResourceCheck):
    def __init__(self) -> None:
        # Check to see the sku is set to premium with any replication block set

        name = "Ensure geo-replicated container registries to match multi-region container deployments."
        id = "CKV_AZURE_165"
        supported_resources = ("azurerm_container_registry",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        sku = conf.get("sku")
        if sku == ["Premium"]:
            replication = conf.get("georeplications")
            if replication:
                return CheckResult.PASSED

        return CheckResult.FAILED


check = ACRGeoreplicated()



from checkov.common.vcs.vcs_schema import VCSSchema


class BranchProtectionSchema(VCSSchema):
    def __init__(self) -> None:
        schema = {
            "$schema": "http://json-schema.org/draft-04/schema#",
            "type": "object",
            "properties": {
                "url": {
                    "type": "string"
                },
                "required_signatures": {
                    "type": "object",
                    "properties": {
                        "url": {
                            "type": "string"
                        },
                        "enabled": {
                            "type": "boolean"
                        }
                    },
                    "required": [
                        "url",
                        "enabled"
                    ]
                },
                "enforce_admins": {
                    "type": "object",
                    "properties": {
                        "url": {
                            "type": "string"
                        },
                        "enabled": {
                            "type": "boolean"
                        }
                    },
                    "required": [
                        "url",
                        "enabled"
                    ]
                },
                "required_linear_history": {
                    "type": "object",
                    "properties": {
                        "enabled": {
                            "type": "boolean"
                        }
                    },
                    "required": [
                        "enabled"
                    ]
                },
                "allow_force_pushes": {
                    "type": "object",
                    "properties": {
                        "enabled": {
                            "type": "boolean"
                        }
                    },
                    "required": [
                        "enabled"
                    ]
                },
                "allow_deletions": {
                    "type": "object",
                    "properties": {
                        "enabled": {
                            "type": "boolean"
                        }
                    },
                    "required": [
                        "enabled"
                    ]
                },
                "required_conversation_resolution": {
                    "type": "object",
                    "properties": {
                        "enabled": {
                            "type": "boolean"
                        }
                    },
                    "required": [
                        "enabled"
                    ]
                }
            },
            "required": [
                "url",
                "enforce_admins",
                "required_linear_history",
                "allow_force_pushes",
                "allow_deletions",
            ]
        }
        super().__init__(schema=schema)


schema = BranchProtectionSchema()



from typing import Dict, Any, List

import pytest

from cdk_integration_tests.utils import run_check, load_failed_checks_from_file

LANGUAGE = 'python'


@pytest.fixture(scope="session", autouse=True)
def failed_checks() -> Dict[str, List[Dict[str, Any]]]:
    report_failed_checks = load_failed_checks_from_file(LANGUAGE)
    yield report_failed_checks


def test_CKV_AWS_18_S3BucketLogging(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_18", policy_name="S3BucketLogging", language="python")


def test_CKV_AWS_19_S3BucketEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_19", policy_name="S3BucketEncryption", language="python")


def test_CKV_AWS_21_S3BucketVersioning(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_21", policy_name="S3BucketVersioning", language="python")


def test_CKV_AWS_145_S3BucketKMSEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_145", policy_name="S3BucketKMSEncryption", language="python")


def test_CKV2_AWS_6_S3BucketPublicAccessBlock(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV2_AWS_6", policy_name="S3BucketPublicAccessBlock", language="python")


def test_CKV_AWS_54_S3BlockPublicPolicy(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_54", policy_name="S3BlockPublicPolicy", language="python")


def test_CKV_AWS_26_SNSTopicEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_26", policy_name="SNSTopicEncryption", language="python")


def test_CKV_AWS_20_S3PublicACLRead(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_20", policy_name="S3PublicACLRead", language="python")


def test_CKV_AWS_55_S3IgnorePublicACLs(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_55", policy_name="S3IgnorePublicACLs", language="python")


def test_CKV_AWS_56_S3RestrictPublicBuckets(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_56", policy_name="S3RestrictPublicBuckets", language="python")


def test_CKV_AWS_53_S3BlockPublicACLs(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_53", policy_name="S3BlockPublicACLs", language="python")


def test_CKV_AWS_57_S3PublicACLWrite(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_57", policy_name="S3PublicACLWrite", language="python")


def test_CKV_AWS_115_LambdaFunctionLevelConcurrentExecutionLimit(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_115", policy_name="LambdaFunctionLevelConcurrentExecutionLimit", language="python")


def test_CKV_AWS_116_LambdaDLQConfigured(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_116", policy_name="LambdaDLQConfigured", language="python")


def test_CKV_AWS_28_DynamodbRecovery(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_28", policy_name="DynamodbRecovery", language="python")


def test_CKV_AWS_158_CloudWatchLogGroupKMSKey(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_158", policy_name="CloudWatchLogGroupKMSKey", language="python")


def test_CKV_AWS_3_EBSEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_3", policy_name="EBSEncryption", language="python")


def test_CKV_AWS_120_APIGatewayCacheEnable(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_120", policy_name="APIGatewayCacheEnable", language="python")


def test_CKV_AWS_163_ECRImageScanning(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_163", policy_name="ECRImageScanning", language="python")


def test_CKV_AWS_51_ECRImmutableTags(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_51", policy_name="ECRImmutableTags", language="python")


def test_CKV_AWS_44_NeptuneClusterStorageEncrypted(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_44", policy_name="NeptuneClusterStorageEncrypted", language="python")


def test_CKV_AWS_166_BackupVaultEncrypted(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_166", policy_name="BackupVaultEncrypted", language="python")


def test_CKV_AWS_74_DocDBEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_74", policy_name="DocDBEncryption", language="python")


def test_CKV_AWS_47_DAXEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_47", policy_name="DAXEncryption", language="python")


def test_CKV_AWS_156_WorkspaceRootVolumeEncrypted(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_156", policy_name="WorkspaceRootVolumeEncrypted", language="python")


def test_CKV_AWS_155_WorkspaceUserVolumeEncrypted(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_155", policy_name="WorkspaceUserVolumeEncrypted", language="python")


def test_CKV_AWS_165_DynamodbGlobalTableRecovery(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_165", policy_name="DynamodbGlobalTableRecovery", language="python")


def test_CKV_AWS_27_SQSQueueEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_27", policy_name="SQSQueueEncryption", language="python")


def test_CKV_AWS_195_GlueSecurityConfigurationEnabled(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_195", policy_name="GlueSecurityConfigurationEnabled", language="python")


def test_CKV_AWS_30_ElasticacheReplicationGroupEncryptionAtTransit(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_30", policy_name="ElasticacheReplicationGroupEncryptionAtTransit", language="python")


def test_CKV_AWS_29_ElasticacheReplicationGroupEncryptionAtRest(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_29", policy_name="ElasticacheReplicationGroupEncryptionAtRest", language="python")


def test_CKV_AWS_43_KinesisStreamEncryptionType(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_43", policy_name="KinesisStreamEncryptionType", language="python")


def test_CKV_AWS_42_EFSEncryptionEnabled(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_42", policy_name="EFSEncryptionEnabled", language="python")


def test_CKV_AWS_193_AppSyncLogging(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_193", policy_name="AppSyncLogging", language="python")


def test_CKV_AWS_194_AppSyncFieldLevelLogs(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_194", policy_name="AppSyncFieldLevelLogs", language="python")


def test_CKV_AWS_104_DocDBAuditLogs(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_104", policy_name="DocDBAuditLogs", language="python")


def test_CKV_AWS_82_AthenaWorkgroupConfiguration(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_82", policy_name="AthenaWorkgroupConfiguration", language="python")


def test_CKV_AWS_17_RDSPubliclyAccessible(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_17", policy_name="RDSPubliclyAccessible", language="python")


def test_CKV_AWS_87_RedshiftClusterPubliclyAccessible(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_87", policy_name="RedshiftClusterPubliclyAccessible", language="python")


def test_CKV_AWS_69_AmazonMQBrokerPublicAccess(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_69", policy_name="AmazonMQBrokerPublicAccess", language="python")


def test_CKV_AWS_118_RDSEnhancedMonitorEnabled(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_118", policy_name="RDSEnhancedMonitorEnabled", language="python")


def test_CKV_AWS_40_IAMPolicyAttachedToGroupOrRoles(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_40", policy_name="IAMPolicyAttachedToGroupOrRoles", language="python")


def test_CKV_AWS_36_CloudTrailLogValidation(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_36", policy_name="CloudTrailLogValidation", language="python")


def test_CKV_AWS_83_ElasticsearchDomainEnforceHTTPS(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_83", policy_name="ElasticsearchDomainEnforceHTTPS", language="python")


def test_CKV_AWS_76_APIGatewayAccessLogging(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_76", policy_name="APIGatewayAccessLogging", language="python")


def test_CKV_AWS_117_LambdaInVPC(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_117", policy_name="LambdaInVPC", language="python")


def test_CKV_AWS_68_WAFEnabled(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_68", policy_name="WAFEnabled", language="python")


def test_CKV_AWS_64_RedshiftClusterEncryption(failed_checks):
    run_check(check_results=failed_checks, check_id="CKV_AWS_64", policy_name="RedshiftClusterEncryption", language="python")



from __future__ import annotations

import os
from enum import Enum
from typing import Iterable, Callable, Any

from checkov.arm.parser.parser import parse
from checkov.common.runners.base_runner import filter_ignored_paths

ARM_POSSIBLE_ENDINGS = [".json"]


class ArmElements(str, Enum):
    OUTPUTS = "outputs"
    PARAMETERS = "parameters"
    RESOURCES = "resources"
    VARIABLES = "variables"

    def __str__(self) -> str:
        # needed, because of a Python 3.11 change
        return self.value


def get_scannable_file_paths(root_folder: str | None = None, excluded_paths: list[str] | None = None) -> set[str]:
    """Finds ARM files"""

    file_paths: "set[str]" = set()
    if not root_folder:
        return file_paths

    for root, d_names, f_names in os.walk(root_folder):
        filter_ignored_paths(root, d_names, excluded_paths)
        filter_ignored_paths(root, f_names, excluded_paths)
        for file in f_names:
            file_ending = os.path.splitext(file)[1]
            if file_ending in ARM_POSSIBLE_ENDINGS:
                file_paths.add(os.path.join(root, file))

    return file_paths


def get_files_definitions(
    files: Iterable[str], filepath_fn: Callable[[str], str] | None = None
) -> tuple[dict[str, dict[str, Any]], dict[str, list[tuple[int, str]]], list[str]]:
    """Parses ARM files into its definitions and raw data"""

    definitions = {}
    definitions_raw = {}
    parsing_errors = []

    for file in files:
        result = parse(file)

        definition, definition_raw = result
        if definition is not None and definition_raw is not None:  # this has to be a 'None' check
            path = filepath_fn(file) if filepath_fn else file
            definitions[path] = definition
            definitions_raw[path] = definition_raw
        else:
            parsing_errors.append(os.path.normpath(file))

    return definitions, definitions_raw, parsing_errors



from __future__ import annotations

from typing import Any

from checkov.common.images.image_referencer import Image
from checkov.azure_pipelines.common.resource_id_utils import generate_resource_key_recursive
from checkov.common.images.workflow.image_referencer_provider import WorkflowImageReferencerProvider


class AzurePipelinesProvider(WorkflowImageReferencerProvider):

    def __init__(self, workflow_config: dict[str, Any], file_path: str):
        super().__init__(workflow_config, file_path)
        self.supported_keys = "container"

    def extract_images_from_workflow(self) -> list[Image]:
        """
        We use a general extraction in azure_pipelines because we can display images in different ways:
            - container
            - container.image
            - resources.containers[].container...
            - jobs[].container...
            - jobs[].steps.task.inputs.container...
            - jobs[].strategy.container...
            - stages[].jobs[]...
        """
        images = self.extract_images_from_dict(self.workflow_config)
        return images

    def extract_images_from_list(self, objects_list: list[dict[str, Any]]) -> list[Image]:
        images = []
        for job in objects_list:
            if isinstance(job, dict):
                images.extend(self.extract_images_from_dict(job))
            if isinstance(job, list):
                images.extend(self.extract_images_from_list(job))
        return images

    def extract_images_from_dict(self, job: dict[str, Any]) -> list[Image]:
        images = []
        start_line, end_line = AzurePipelinesProvider._get_start_end_lines(job)
        for key, sub_job in job.items():
            if key == self.supported_keys:
                image = self.create_image(sub_job, start_line, end_line)
                if image:
                    images.append(image)
            elif isinstance(sub_job, dict):
                images.extend(self.extract_images_from_dict(sub_job))
            elif isinstance(sub_job, list):
                images.extend(self.extract_images_from_list(sub_job))
        return images

    def create_image(self, container: dict[str, Any] | str, start_line: int, end_line: int) -> Image | None:
        image_name = ''
        if isinstance(container, str):
            image_name = container
        elif isinstance(container, dict):
            if 'image' in container:
                image_name = container['image']
        related_resource_id = generate_resource_key_recursive(file_conf=self.workflow_config,
                                                              resource_key='',
                                                              start_line=start_line,
                                                              end_line=end_line)
        if image_name and related_resource_id:
            return Image(
                file_path=self.file_path,
                name=image_name,
                start_line=start_line,
                end_line=end_line,
                related_resource_id=related_resource_id)
        return None



from typing import Any

from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class APIServicesUseVirtualNetwork(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that API management services use virtual networks"
        id = "CKV_AZURE_107"
        supported_resources = ("Microsoft.ApiManagement/service",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            missing_block_result=CheckResult.FAILED,
        )

    def get_inspected_key(self) -> str:
        return "properties/virtualNetworkConfiguration"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = APIServicesUseVirtualNetwork()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class EC2DetailedMonitoringEnabled(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure that detailed monitoring is enabled for EC2 instances"
        id = "CKV_AWS_126"
        supported_resources = ['aws_instance']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'monitoring'

    def get_expected_value(self):
        return True


check = EC2DetailedMonitoringEnabled()



from typing import Set, List

from checkov.common.bridgecrew.severities import Severity
from checkov.common.sast.consts import SastLanguages
import sys
if sys.version_info < (3, 11):
    from typing_extensions import TypedDict, NotRequired
else:
    from typing import TypedDict, NotRequired


class LibraryInput(TypedDict):
    languages: Set[SastLanguages]
    source_codes: List[str]
    policies: List[str]
    checks: List[str]
    skip_checks: List[str]
    skip_path: List[str]
    check_threshold: Severity
    skip_check_threshold: Severity
    list_policies: NotRequired[bool]
    report_imports: bool
    remove_default_policies: NotRequired[bool]
    report_reachability: bool



from typing import Dict, Any, Set

from pydantic import BaseModel


class ReachabilityData(BaseModel):
    aliasMapping: Dict[str, Any]  # noqa: CCE003


class ReachabilityRunConfig(BaseModel):
    packageNamesForMapping: Set[str]  # noqa: CCE003



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class DocDBBackupRetention(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure DocDB has an adequate backup retention period"
        id = "CKV_AWS_360"
        supported_resources = ['aws_docdb_cluster']
        categories = [CheckCategories.BACKUP_AND_RECOVERY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "backup_retention_period"

    def scan_resource_conf(self, conf):
        if conf.get("backup_retention_period", [1])[0] >= 7:
            return CheckResult.PASSED
        return CheckResult.FAILED


check = DocDBBackupRetention()



from checkov.terraform.checks.resource.alicloud.AbsRDSParameter import AbsRDSParameter


class RDSInstanceLogConnections(AbsRDSParameter):
    def __init__(self):
        super().__init__(check_id="CKV_ALI_37", parameter="log_connections")


check = RDSInstanceLogConnections()



from __future__ import annotations

import logging
from pathlib import Path
from typing import TYPE_CHECKING

from _pytest.logging import LogCaptureFixture
from typing_extensions import Literal

from checkov import main
from checkov.common.runners.base_runner import BaseRunner
from checkov.common.runners.runner_registry import RunnerRegistry
from checkov.main import DEFAULT_RUNNERS, Checkov
from checkov.runner_filter import RunnerFilter

if TYPE_CHECKING:
    import argparse
    from checkov.common.output.baseline import Baseline
    from checkov.common.output.report import Report


class CustomRunnerRegistry(RunnerRegistry):
    def __init__(self, banner: str, runner_filter: RunnerFilter, *runners: BaseRunner) -> None:
        super().__init__(banner, runner_filter, *runners)

    def print_reports(
        self,
        scan_reports: list[Report],
        config: argparse.Namespace,
        url: str | None = None,
        created_baseline_path: str | None = None,
        baseline: Baseline | None = None,
    ) -> Literal[0, 1]:
        # result doesn't matter, just don't want it to print to console
        return 0


def test_run_with_outer_registry_and_framework_flag():
    # given
    custom_banner = "custom banner"
    resource_dir = Path(__file__).parent / "common/runner_registry/example_multi_iac"
    argv = ["-d", str(resource_dir), "--framework", "terraform"]

    # when
    main.outer_registry = CustomRunnerRegistry(custom_banner, RunnerFilter(), *DEFAULT_RUNNERS)
    ckv = Checkov()
    ckv.parse_config(argv=argv)
    ckv.run(banner=custom_banner)

    # then
    assert len(main.outer_registry.runners) == 1
    assert main.outer_registry.runners[0].check_type == "terraform"

    # cleanup
    main.outer_registry = None


def test_run():
    # given
    custom_banner = "custom banner"
    custom_tool = "custom tool"
    resource_dir = Path(__file__).parent / "common/runner_registry/example_multi_iac"
    argv = ["-d", str(resource_dir), "--framework", "terraform", "kubernetes"]

    # when
    ckv = Checkov()
    ckv.parse_config(argv=argv)
    ckv.run(banner=custom_banner, tool=custom_tool)

    # then
    # check run_metadata has all fields set
    assert ckv.run_metadata["checkov_version"] and isinstance(ckv.run_metadata["checkov_version"], str)
    assert ckv.run_metadata["python_executable"] and isinstance(ckv.run_metadata["python_executable"], str)
    assert ckv.run_metadata["python_version"] and isinstance(ckv.run_metadata["python_version"], str)
    assert ckv.run_metadata["checkov_executable"] and isinstance(ckv.run_metadata["checkov_executable"], str)
    assert ckv.run_metadata["args"] and isinstance(ckv.run_metadata["args"], list)

    # check all runners were initialized, but only 2 were actually run
    assert len(ckv.runners) == 29

    assert len(ckv.scan_reports) == 2
    assert {report.check_type for report in ckv.scan_reports} == {"kubernetes", "terraform"}


def test_run_with_severity_filter_and_api_key(caplog: LogCaptureFixture):
    # given
    caplog.set_level(logging.WARNING)
    custom_banner = "custom banner"
    resource_dir = Path(__file__).parent / "common/runner_registry/example_multi_iac"
    argv = [
        "-d", str(resource_dir),
        "--framework", "terraform",
        "--check", "MEDIUM",
        "--bc-api-key", "12345678-abcd-1234-abcd-123456789012",
        "--repo-id", "acme/example",
        "--show-config",  # just set to terminate the run early enough
    ]

    # when
    ckv = Checkov()
    ckv.parse_config(argv=argv)
    ckv.run(banner=custom_banner)

    # then
    assert not caplog.messages


def test_run_with_severity_filter_without_api_key(caplog: LogCaptureFixture):
    # given
    caplog.set_level(logging.WARNING)
    custom_banner = "custom banner"
    resource_dir = Path(__file__).parent / "common/runner_registry/example_multi_iac"
    argv = [
        "-d", str(resource_dir),
        "--framework", "terraform",
        "--check", "MEDIUM",
    ]

    # when
    ckv = Checkov()
    ckv.parse_config(argv=argv)
    ckv.run(banner=custom_banner)

    # then
    assert caplog.messages == ["Filtering checks by severity is only possible with an API key"]


def test_run_with_severity_skip_filter_without_api_key(caplog: LogCaptureFixture):
    # given
    caplog.set_level(logging.WARNING)
    custom_banner = "custom banner"
    resource_dir = Path(__file__).parent / "common/runner_registry/example_multi_iac"
    argv = [
        "-d", str(resource_dir),
        "--framework", "terraform",
        "--skip-check", "MEDIUM",
    ]

    # when
    ckv = Checkov()
    ckv.parse_config(argv=argv)
    ckv.run(banner=custom_banner)

    # then
    assert caplog.messages == ["Filtering checks by severity is only possible with an API key"]



from __future__ import annotations

import json
import re
from typing import List, Any

from cloudsplaining.scan.resource_policy_document import ResourcePolicyDocument

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck

DATA_TO_JSON_PATTERN = re.compile(r"\$?\{?(.+?)(?=.json).json\}?")


class GlacierVaultAnyPrincipal(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Glacier Vault access policy is not public by only allowing specific services or principals to access it"
        id = "CKV_AWS_167"
        supported_resources = ("aws_glacier_vault",)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if "access_policy" not in conf:
            return CheckResult.PASSED
        policy_obj = conf["access_policy"][0]
        if isinstance(policy_obj, str):
            if re.match(DATA_TO_JSON_PATTERN, policy_obj):
                return CheckResult.UNKNOWN
            else:
                try:
                    policy_obj = json.loads(policy_obj)
                except Exception:
                    return CheckResult.UNKNOWN
        try:
            policy = ResourcePolicyDocument(policy=policy_obj)
            if policy.internet_accessible_actions:
                return CheckResult.FAILED
        except (TypeError, AttributeError):
            return CheckResult.UNKNOWN
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ["access_policy"]


check = GlacierVaultAnyPrincipal()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class KeyBackedByHSM(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that key vault key is backed by HSM"
        id = "CKV_AZURE_112"
        supported_resources = ("Microsoft.KeyVault/vaults/keys",)
        categories = (CheckCategories.BACKUP_AND_RECOVERY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "properties/kty"

    def get_expected_value(self) -> Any:
        return "RSA-HSM"

    def get_expected_values(self) -> list[Any]:
        return [self.get_expected_value(), "EC-HSM"]


check = KeyBackedByHSM()



from checkov.terraform.checks.resource.openstack.AbsSecurityGroupUnrestrictedIngress import AbsSecurityGroupUnrestrictedIngress


class SecurityGroupUnrestrictedIngress22(AbsSecurityGroupUnrestrictedIngress):
    def __init__(self):
        super().__init__(check_id="CKV_OPENSTACK_2", port=22)


check = SecurityGroupUnrestrictedIngress22()



from __future__ import annotations

from typing import Any
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.checks.enums import BlockType
from checkov.openapi.checks.resource.v2.BaseOpenapiCheckV2 import BaseOpenapiCheckV2


class Oauth2SecurityPasswordFlow(BaseOpenapiCheckV2):
    def __init__(self) -> None:
        id = "CKV_OPENAPI_8"
        name = "Ensure that security is not using 'password' flow in OAuth2 authentication - version 2.0 files"
        categories = [CheckCategories.API_SECURITY]
        supported_resources = ["security"]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_resources,
            block_type=BlockType.DOCUMENT,
        )

    def scan_openapi_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]]:
        security_values = conf.get("security") or [{}]
        security_definitions = conf.get('securityDefinitions') or {}

        for auth_dict in security_values:
            if not isinstance(auth_dict, dict):
                return CheckResult.UNKNOWN, conf
            for auth_key in auth_dict:
                if self.is_start_end_line(auth_key):
                    continue
                if not isinstance(security_definitions, dict):
                    return CheckResult.UNKNOWN, conf
                auth_definition = security_definitions.get(auth_key, {})
                auth_type = auth_definition.get('type', '')
                if auth_type.lower() == 'oauth2':
                    auth_flow = auth_definition.get('flow', '')
                    if auth_flow == 'password':
                        return CheckResult.FAILED, auth_dict

        return CheckResult.PASSED, conf


check = Oauth2SecurityPasswordFlow()



from __future__ import annotations
import abc

from checkov.common.output.record import Record
from checkov.policies_3d.syntax.syntax import Predicate
from typing import Any


class IACPredicate(Predicate):
    def __init__(self, record: Record) -> None:
        super().__init__()
        self.record = record

    @abc.abstractmethod
    def __call__(self) -> bool:
        raise NotImplementedError()


class ViolationIdEquals(IACPredicate):
    def __init__(self, record: Record, violation_id: str) -> None:
        super().__init__(record)
        self.violation_id = violation_id

    def __call__(self) -> bool:
        self.is_true = isinstance(self.violation_id, str) and self.record.bc_check_id == self.violation_id
        return self.is_true

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, ViolationIdEquals):
            return False

        return self.violation_id == other.violation_id and self.record.bc_check_id == other.record.bc_check_id

    def __hash__(self) -> Any:
        return hash(('violation_id', self.violation_id, 'bc_check_id', self.record.bc_check_id))



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class FunctionAppMinTLSVersion(BaseResourceValueCheck):
    def __init__(self):
        """
        The minimum supported TLS version for the function app.
        Defaults to 1.2 for new function apps.
        field name is:
         - min_tls_version in azurerm_function_app, azurerm_function_app_slot.
         - minimum_tls_version in newer resources (with linux/windows).
        """
        name = "Ensure Function app is using the latest version of TLS encryption"
        id = "CKV_AZURE_145"
        supported_resources = ['azurerm_function_app', 'azurerm_linux_function_app', 'azurerm_windows_function_app',
                               'azurerm_function_app_slot', 'azurerm_linux_function_app_slot',
                               'azurerm_windows_function_app_slot']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self):
        if self.entity_type in ("azurerm_function_app", "azurerm_function_app_slot"):
            return "site_config/[0]/min_tls_version"
        else:
            return "site_config/[0]/minimum_tls_version"

    def get_expected_value(self):
        return 1.2

    def get_expected_values(self):
        return ["1.2", 1.2]


check = FunctionAppMinTLSVersion()



from __future__ import annotations

from typing import TYPE_CHECKING, Union

from checkov.common.images.graph.image_referencer_manager import GraphImageReferencerManager
from checkov.helm.image_referencer.provider.helm import HelmProvider

if TYPE_CHECKING:
    from checkov.common.images.image_referencer import Image
    from networkx import DiGraph
    from igraph import Graph


class HelmImageReferencerManager(GraphImageReferencerManager):

    def __init__(self, graph_connector: Union[Graph, DiGraph], original_root_dir: str, temp_root_dir: str):
        super().__init__(graph_connector)
        self.original_root_dir = original_root_dir
        self.temp_root_dir = temp_root_dir

    def extract_images_from_resources(self) -> list[Image]:
        helm_provider = HelmProvider(graph_connector=self.graph_connector, original_root_dir=self.original_root_dir,
                                     temp_root_dir=self.temp_root_dir)
        images = helm_provider.extract_images_from_resources()

        return images



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AppServiceEnvironmentZoneRedundant(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure App Service Environment is zone redundant"
        id = "CKV_AZURE_231"
        supported_resources = ("azurerm_app_service_environment_v3",)
        categories = (CheckCategories.BACKUP_AND_RECOVERY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "zone_redundant"


check = AppServiceEnvironmentZoneRedundant()



from __future__ import annotations

import logging
import os
from collections.abc import Iterable
from typing import TYPE_CHECKING, Any

from typing_extensions import TypeAlias  # noqa[TC002]

from checkov.common.checks_infra.registry import get_graph_checks_registry
from checkov.common.models.enums import CheckResult
from checkov.common.typing import LibraryGraphConnector
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.graph.graph_builder.consts import GraphSource
from checkov.common.images.image_referencer import ImageReferencerMixin
from checkov.common.output.record import Record
from checkov.common.output.report import Report
from checkov.common.output.extra_resource import ExtraResource
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.runners.base_runner import BaseRunner
from checkov.common.util.consts import START_LINE, END_LINE
from checkov.common.util.dockerfile import is_dockerfile
from checkov.common.typing import _CheckResult
from checkov.dockerfile.graph_builder.local_graph import DockerfileLocalGraph
from checkov.dockerfile.graph_manager import DockerfileGraphManager
from checkov.dockerfile.image_referencer.manager import DockerfileImageReferencerManager
from checkov.dockerfile.parser import collect_skipped_checks
from checkov.dockerfile.registry import registry
from checkov.dockerfile.utils import (
    DOCKERFILE_STARTLINE,
    DOCKERFILE_ENDLINE,
    get_files_definitions,
    get_scannable_file_paths,
    get_abs_path,
    build_definitions_context,
)
from checkov.runner_filter import RunnerFilter

if TYPE_CHECKING:
    from dockerfile_parse.parser import _Instruction  # only in extra_stubs
    from networkx import DiGraph
    from checkov.common.checks.base_check import BaseCheck
    from checkov.common.graph.checks_infra.base_check import BaseGraphCheck
    from checkov.common.images.image_referencer import Image

_DockerfileContext: TypeAlias = "dict[str, dict[str, Any]]"
_DockerfileDefinitions: TypeAlias = "dict[str, dict[str, list[_Instruction]]]"


class Runner(ImageReferencerMixin[_DockerfileDefinitions], BaseRunner[_DockerfileDefinitions, _DockerfileContext, DockerfileGraphManager]):
    check_type = CheckType.DOCKERFILE  # noqa: CCE003  # a static attribute

    def __init__(
        self,
        db_connector: LibraryGraphConnector | None = None,
        source: str = GraphSource.DOCKERFILE,
        graph_class: type[DockerfileLocalGraph] = DockerfileLocalGraph,
        graph_manager: DockerfileGraphManager | None = None,
    ) -> None:

        super().__init__()
        db_connector = db_connector or self.db_connector
        self.graph_class = graph_class
        self.graph_manager = (
            graph_manager if graph_manager else DockerfileGraphManager(source=source, db_connector=db_connector)
        )
        self.graph_registry = get_graph_checks_registry(self.check_type)

        self.context: _DockerfileContext = {}
        self.definitions: _DockerfileDefinitions = {}
        self.definitions_raw: "dict[str, list[str]]" = {}       # type:ignore[assignment]
        self.root_folder: str | None = None

    def should_scan_file(self, filename: str) -> bool:
        return is_dockerfile(os.path.basename(filename))

    def run(
        self,
        root_folder: str | None = None,
        external_checks_dir: list[str] | None = None,
        files: list[str] | None = None,
        runner_filter: RunnerFilter | None = None,
        collect_skip_comments: bool = True,
    ) -> Report | list[Report]:
        runner_filter = runner_filter or RunnerFilter()
        if not runner_filter.show_progress_bar:
            self.pbar.turn_off_progress_bar()

        report = Report(self.check_type)

        if not self.context or not self.definitions:
            files_list: "Iterable[str]" = []
            filepath_fn = None
            if external_checks_dir:
                for directory in external_checks_dir:
                    registry.load_external_checks(directory)

                    if self.graph_registry:
                        self.graph_registry.load_external_checks(directory)

            if files:
                files_list = [file for file in files if is_dockerfile(os.path.basename(file))]

            if root_folder:
                filepath_fn = lambda f: f"/{os.path.relpath(f, os.path.commonprefix((root_folder, f)))}"
                self.root_folder = root_folder

                files_list = get_scannable_file_paths(root_folder=root_folder, excluded_paths=runner_filter.excluded_paths)

            self.definitions, self.definitions_raw = get_files_definitions(files_list, filepath_fn)

            self.context = build_definitions_context(definitions=self.definitions, definitions_raw=self.definitions_raw)

            if self.graph_registry and self.graph_manager:
                logging.info("Creating Dockerfile graph")
                local_graph = self.graph_manager.build_graph_from_definitions(definitions=self.definitions)
                logging.info("Successfully created Dockerfile graph")

                self.graph_manager.save_graph(local_graph)

        self.pbar.initiate(len(self.definitions))

        # run Python checks
        self.add_python_check_results(report=report, runner_filter=runner_filter, root_folder=root_folder)

        # run graph checks
        if self.graph_registry:
            self.add_graph_check_results(report=report, runner_filter=runner_filter, root_folder=root_folder)

        if runner_filter.run_image_referencer:
            if files:
                # 'root_folder' shouldn't be empty to remove the whole path later and only leave the shortened form
                root_folder = os.path.split(os.path.commonprefix(files))[0]

            image_report = self.check_container_image_references(
                root_path=root_folder,
                runner_filter=runner_filter,
                definitions=self.definitions,
            )

            if image_report:
                # due too many tests failing only return a list, if there is an image report
                return [report, image_report]

        return report

    def add_python_check_results(self, report: Report, runner_filter: RunnerFilter, root_folder: str | None) -> None:
        """Adds Python check results to given report"""

        for dockerfile_path, instructions in self.definitions.items():
            self.pbar.set_additional_data({"Current File Scanned": os.path.relpath(dockerfile_path, root_folder)})

            file_abs_path = get_abs_path(root_folder=root_folder, file_path=dockerfile_path)
            report.add_resource(file_abs_path)
            skipped_checks = collect_skipped_checks(instructions)

            results = registry.scan(dockerfile_path, instructions, skipped_checks, runner_filter)

            if results:
                for check, check_result in results.items():
                    result_configuration = check_result["results_configuration"]
                    startline = 0
                    endline = len(self.definitions_raw[dockerfile_path]) - 1
                    result_instruction = ""
                    if result_configuration:
                        if isinstance(result_configuration, list):
                            for res in result_configuration:
                                startline = res[DOCKERFILE_STARTLINE]
                                endline = res[DOCKERFILE_ENDLINE]
                                result_instruction = res["instruction"]
                                self.build_record(
                                    report,
                                    self.definitions_raw,
                                    dockerfile_path,
                                    file_abs_path,
                                    check,
                                    check_result,
                                    startline,
                                    endline,
                                    result_instruction,
                                )
                        else:
                            startline = result_configuration[DOCKERFILE_STARTLINE]
                            endline = result_configuration[DOCKERFILE_ENDLINE]
                            result_instruction = result_configuration["instruction"]
                            self.build_record(
                                report,
                                self.definitions_raw,
                                dockerfile_path,
                                file_abs_path,
                                check,
                                check_result,
                                startline,
                                endline,
                                result_instruction,
                            )
                    else:
                        self.build_record(
                            report,
                            self.definitions_raw,
                            dockerfile_path,
                            file_abs_path,
                            check,
                            check_result,
                            startline,
                            endline,
                            result_instruction,
                        )
            else:
                report.extra_resources.add(
                    ExtraResource(
                        file_abs_path=file_abs_path,
                        file_path=dockerfile_path,
                        resource=dockerfile_path,
                    )
                )

            self.pbar.update()
        self.pbar.close()

    def add_graph_check_results(self, report: Report, runner_filter: RunnerFilter, root_folder: str | None) -> None:
        """Adds graph check results to given report"""

        graph_checks_results = self.run_graph_checks_results(runner_filter, self.check_type)

        for check, check_results in graph_checks_results.items():
            for check_result in check_results:
                entity = check_result["entity"]
                entity_file_path: str = entity[CustomAttributes.FILE_PATH]
                file_abs_path = get_abs_path(root_folder=root_folder, file_path=entity_file_path)
                resource_type: str = entity[CustomAttributes.RESOURCE_TYPE]
                start_line = entity[START_LINE]
                end_line = entity[END_LINE]

                self.build_record(
                    report=report,
                    definitions_raw=self.definitions_raw,
                    dockerfile_path=entity_file_path,
                    file_abs_path=file_abs_path,
                    check=check,
                    check_result=check_result,
                    startline=start_line,
                    endline=end_line,
                    result_instruction=resource_type,
                )

    def calc_record_codeblock(
        self,
        codeblock: list[tuple[int, str]],
        definitions_raw: dict[str, list[str]],
        dockerfile_path: str,
        endline: int,
        startline: int,
    ) -> None:
        for line in range(startline, endline + 1):
            codeblock.append((line + 1, definitions_raw[dockerfile_path][line]))

    def build_record(
        self,
        report: Report,
        definitions_raw: dict[str, list[str]],
        dockerfile_path: str,
        file_abs_path: str,
        check: BaseCheck | BaseGraphCheck,
        check_result: _CheckResult,
        startline: int,
        endline: int,
        result_instruction: str,
    ) -> None:
        codeblock: list[tuple[int, str]] = []

        if result_instruction:
            entity_context = next(
                (
                    resource
                    for resource in self.context[dockerfile_path][result_instruction]
                    if resource["start_line"] == startline + 1
                ),
                None,
            )
            if entity_context:
                codeblock = entity_context["code_lines"]
                skipped_check = next(
                    (
                        skipped_check
                        for skipped_check in entity_context.get("skipped_checks", [])
                        if skipped_check["id"] in (check.id, check.bc_id)
                    ),
                    None,
                )
                if skipped_check:
                    check_result["result"] = CheckResult.SKIPPED
                    check_result["suppress_comment"] = skipped_check.get("suppress_comment", "")
            else:
                logging.info(f"Could not find context for resource with start line {startline + 1} in {self.context[dockerfile_path][result_instruction]}")
        else:
            self.calc_record_codeblock(codeblock, definitions_raw, dockerfile_path, endline, startline)

        record = Record(
            check_id=check.id,
            bc_check_id=check.bc_id,
            check_name=check.name,
            check_result=check_result,
            code_block=codeblock,
            file_path=dockerfile_path,
            file_line_range=[startline + 1, endline + 1],
            resource=f"{dockerfile_path}.{result_instruction}",
            evaluations=None,
            check_class=check.__class__.__module__,
            file_abs_path=file_abs_path,
            entity_tags=None,
            severity=check.severity,
        )
        record.set_guideline(check.guideline)
        report.add_record(record=record)

    def extract_images(
        self,
        graph_connector: DiGraph | None = None,
        definitions: dict[str, dict[str, list[_Instruction]]] | None = None,
        definitions_raw: dict[str, list[tuple[int, str]]] | None = None,
    ) -> list[Image]:
        if not definitions:
            # should not happen
            return []

        manager = DockerfileImageReferencerManager(definitions=definitions)
        images = manager.extract_images_from_resources()

        return images



import asyncio
import logging
from typing import Set, Dict, Any, Union
from pydantic import ValidationError
from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.sca.reachability.package_alias_mapping.alias_mapping_creator import AliasMappingCreator
from .models import ReachabilityRunConfig, ReachabilityData

RELEVANT_PACKAGES = {"Mongoose", "Node-etsy-client", "semver", "moment", "sails", "protobufjs", "qs", "@angular/core",
                     "minimist", "ansi-regex", "minimatch", "request", "axios", "lodash", "http-cache-semantics",
                     "node-sass", "got", "jquery", "file-type", "bootstrap", "tough-cookie", "aws-sdk-js-v3",
                     "aws-sdk-js", "tar", "json-schema", "path-parse", "decode-uri-component", "angular",
                     "hosted-git-info", "ajv", "y18n", "ini", "materialize-css", "shelljs", "async", "ssri", "xml2js",
                     "chart.js", "validator", "glob-parent", "npm/cli", "postcss", "gatsby-plugin-sharp", "json5",
                     "xmldom", "debug", "terser", "brace-expansion", "sshpk", "is-my-json-valid", "ms", "jsonpointer",
                     "bl", "fstream", "npm-user-validate", "chownr", "hawk", "extend", "@hapi/hoek", "@hapi/cryptiles",
                     "casperjs", "marked", "karma", "swig-templates", "bson", "word-wrap", "fresh", "pug", "dicer"}


class SastReachabilityDataFetcher:
    def __init__(self) -> None:
        self.alias_mapping_creator = AliasMappingCreator()
        self.reachability_run_config_raw: Union[Dict[str, Any], None] = None
        self.reachability_run_config: Union[ReachabilityRunConfig, None] = None

    def fetch(self, repository_name: str, repository_root_dir: str) -> Union[ReachabilityData, None]:
        self.reachability_run_config_raw = bc_integration.get_reachability_run_config()

        if not self.reachability_run_config_raw:
            logging.error('reachability_run_config is null, unable to proceed', exc_info=True)
            return None

        try:
            self.reachability_run_config = ReachabilityRunConfig(packageNamesForMapping=RELEVANT_PACKAGES)
        except ValidationError:
            logging.error('Unable to serialize reachability run_config', exc_info=True)
            return None

        try:
            result = ReachabilityData(
                aliasMapping=self._fetch_alias_mapping(repository_name=repository_name,
                                                       repository_root_dir=repository_root_dir,
                                                       relevant_packages=self.reachability_run_config.packageNamesForMapping)
            )
        except ValidationError:
            logging.error('Unable to serialize reachability data', exc_info=True)
            return None

        return result

    def _fetch_alias_mapping(self, repository_name: str, repository_root_dir: str, relevant_packages: Set[str]) -> Dict[str, Any]:
        self.alias_mapping_creator.update_alias_mapping_for_repository(
            repository_name=repository_name,
            repository_root_dir=repository_root_dir,
            relevant_packages=relevant_packages
        )
        res: Dict[str, Any] = self.alias_mapping_creator.get_alias_mapping()
        asyncio.run(bc_integration.persist_reachability_alias_mapping(res))
        return res



from __future__ import annotations

from typing import Any

from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.data_structures_utils import find_in_dict


class AppServiceHttps20Enabled(BaseResourceCheck):
    # apiVersion = 2018-11-01 - http20Enabled is a string
    # apiVersion > 2020-10-01  - http20Enabled is a boolean
    def __init__(self) -> None:
        name = "Ensure that 'HTTP Version' is the latest if used to run the web app"
        id = "CKV_AZURE_18"
        supported_resources = ("Microsoft.Web/sites",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        http_20_enabled = find_in_dict(conf, "properties/siteConfig/http20Enabled")
        if http_20_enabled and "apiVersion" in conf:
            if conf["apiVersion"] == "2018-11-01":
                if isinstance(http_20_enabled, str) and str(http_20_enabled).lower() == "true":
                    return CheckResult.PASSED
            elif isinstance(http_20_enabled, bool) and http_20_enabled:
                return CheckResult.PASSED
        return CheckResult.FAILED


check = AppServiceHttps20Enabled()



from __future__ import annotations
import abc
from typing import Any


class Predicate:
    def __init__(self) -> None:
        self.is_true = False

    @abc.abstractmethod
    def __call__(self, *args: Any, **kwargs: Any) -> bool:
        raise NotImplementedError()

    @abc.abstractmethod
    def __eq__(self, other: object) -> bool:
        raise NotImplementedError()

    @abc.abstractmethod
    def __hash__(self) -> Any:
        raise NotImplementedError()


class Predicament:
    def __init__(self, logical_op: str, predicates: list[Predicate] | None = None,
                 predicaments: list[Predicament] | None = None) -> None:
        self.predicates = predicates or []
        self.predicaments = predicaments or []
        self.logical_op = logical_op.lower()

    def __call__(self) -> bool:
        """
        Calls all direct and indirect predicates.

        Implementation note: using a list comprehension in any/all guarantees that all predicates will be computed
        by avoiding a short-circuit evaluation.
        """
        if self.logical_op == 'or':
            sub_predicaments_result = False
            if self.predicaments:
                sub_predicaments_result = any([predicament() for predicament in self.predicaments])

            return any([predicate() for predicate in self.predicates]) or sub_predicaments_result

        sub_predicaments_result = True
        if self.predicaments:
            sub_predicaments_result = all(predicament() for predicament in self.predicaments)

        return all([predicate() for predicate in self.predicates]) and sub_predicaments_result

    def get_all_children_predicates(self) -> set[Predicate]:
        predicates = set()
        for sub_predicament in self.predicaments:
            predicates.update(sub_predicament.get_all_children_predicates())

        predicates.update(set(self.predicates))
        return predicates



import json
import os

current_dir = os.path.dirname(os.path.realpath(__file__))


def test_sast_python() -> None:
    report_path = os.path.join(current_dir, '..', 'checkov_report_sast_python.json')
    validate_report(os.path.abspath(report_path))


def test_sast_java() -> None:
    report_path = os.path.join(current_dir, '..', 'checkov_report_sast_java.json')
    validate_report(os.path.abspath(report_path))


def test_sast_javascript() -> None:
    report_path = os.path.join(current_dir, '..', 'checkov_report_sast_javascript.json')
    validate_report(os.path.abspath(report_path))


def validate_report(report_path: str) -> None:
    with open(report_path) as f:
        data = f.read()
        report = json.loads(data)
        assert report is not None
        results = report.get("results")
        assert results is not None
        passed_checks = results.get("passed_checks")
        failed_checks = results.get("failed_checks")
        assert not passed_checks
        assert failed_checks is not None
        assert isinstance(failed_checks, list)
        assert len(failed_checks) > 0
        summary = report.get("summary")
        assert summary.get("passed") == 0
        assert summary.get("failed") > 0



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.checks.resource.base_spec_check import BaseK8Check


class TillerService(BaseK8Check):
    def __init__(self) -> None:
        name = "Ensure that the Tiller Service (Helm v2) is deleted"
        id = "CKV_K8S_44"
        # Location: container .image
        supported_kind = ("Service",)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def scan_spec_conf(self, conf: dict[str, Any]) -> CheckResult:
        metadata = conf.get("metadata")
        if metadata:
            if "name" in metadata and "tiller" in str(metadata["name"]).lower():
                return CheckResult.FAILED
            labels = metadata.get("labels")
            if labels and isinstance(labels, dict):
                for v in labels.values():
                    if "tiller" in str(v).lower():
                        return CheckResult.FAILED

        spec = conf.get("spec")
        if spec:
            selector = spec.get("selector")
            if selector and isinstance(selector, dict):
                for v in selector.values():
                    if "tiller" in str(v).lower():
                        return CheckResult.FAILED

        return CheckResult.UNKNOWN


check = TillerService()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck


class DocDBBackupRetention(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure DocDB has an adequate backup retention period"
        id = "CKV_AWS_360"
        supported_resources = ("AWS::DocDB::DBCluster",)
        categories = (CheckCategories.BACKUP_AND_RECOVERY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "Properties/BackupRetentionPeriod"

    def scan_resource_conf(self, conf) -> CheckResult:
        properties = conf.get("Properties")
        if properties:
            backup_retention_period = properties.get("BackupRetentionPeriod", 1)
            if backup_retention_period >= 7:
                return CheckResult.PASSED
        return CheckResult.FAILED


check = DocDBBackupRetention()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck

BAD_CIPHERS = {
    "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384",
    "TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256",
    "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA",
    "TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA",
    "TLS_DHE_RSA_WITH_AES_256_GCM_SHA384",
    "TLS_DHE_RSA_WITH_AES_128_GCM_SHA256",
    "TLS_DHE_RSA_WITH_AES_256_CBC_SHA",
    "TLS_DHE_RSA_WITH_AES_128_CBC_SHA",
    "TLS_RSA_WITH_AES_256_GCM_SHA384",
    "TLS_RSA_WITH_AES_128_GCM_SHA256",
    "TLS_RSA_WITH_AES_256_CBC_SHA256",
    "TLS_RSA_WITH_AES_128_CBC_SHA256",
    "TLS_RSA_WITH_AES_256_CBC_SHA",
    "TLS_RSA_WITH_AES_128_CBC_SHA",
    "TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384 ",
    "TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256",
    "TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA",
    "TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA",
    "TLS_DHE_DSS_WITH_AES_256_CBC_SHA256",
    "TLS_DHE_DSS_WITH_AES_128_CBC_SHA256 ",
    "TLS_DHE_DSS_WITH_AES_256_CBC_SHA",
    "TLS_DHE_DSS_WITH_AES_128_CBC_SHA",
    "TLS_RSA_WITH_3DES_EDE_CBC_SHA",
    "TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA",
}
PROTOCOL_VERSIONS = {"TLSv1_2", "TLSv1_3"}


class AppGWDefinesSecureProtocols(BaseResourceCheck):
    def __init__(self) -> None:
        """
        https://azure.github.io/PSRule.Rules.Azure/en/rules/Azure.AppGw.SSLPolicy/

        """
        name = "Ensure Application Gateway defines secure protocols for in transit communication"
        id = "CKV_AZURE_218"
        supported_resources = ("azurerm_application_gateway",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        ssl_policy = conf.get("ssl_policy")
        if ssl_policy and isinstance(ssl_policy, list):
            ssl_policy = ssl_policy[0]
            policy_type = ssl_policy.get("policy_type")
            if policy_type and isinstance(policy_type, list):
                if policy_type[0] != "Predefined":
                    protocol_version = ssl_policy.get("min_protocol_version")
                    if (
                        protocol_version
                        and isinstance(protocol_version, list)
                        and protocol_version[0] in PROTOCOL_VERSIONS
                    ):
                        ciphers = ssl_policy.get("cipher_suites")
                        if ciphers and isinstance(ciphers, list) and any(cipher in BAD_CIPHERS for cipher in ciphers[0]):
                            return CheckResult.FAILED
                        return CheckResult.PASSED

                policy_name = ssl_policy.get("policy_name")
                if policy_name and isinstance(policy_name, list) and policy_name[0] == "AppGwSslPolicy20220101S":
                    return CheckResult.PASSED
                return CheckResult.FAILED

        return CheckResult.FAILED


check = AppGWDefinesSecureProtocols()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from typing import List, Any


class AppConfigPublicAccess(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure 'Public Access' is not Enabled for App configuration"
        id = "CKV_AZURE_185"
        supported_resources = ("azurerm_app_configuration",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "public_network_access"

    def get_forbidden_values(self) -> List[Any]:
        return ['Enabled']


check = AppConfigPublicAccess()



from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class GKEEnableVPCFlowLogs(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Enable VPC Flow Logs and Intranode Visibility"
        id = "CKV_GCP_61"
        supported_resources = ('google_container_cluster',)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'enable_intranode_visibility'

    def get_expected_value(self) -> Any:
        return True


check = GKEEnableVPCFlowLogs()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class BigTableInstanceDeletionProtection(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Big Table Instances have deletion protection enabled"
        id = "CKV_GCP_122"
        supported_resources = ['google_bigtable_instance']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.FAILED)

    def get_inspected_key(self):
        return 'deletion_protection'

    def get_expected_value(self):
        return True


check = BigTableInstanceDeletionProtection()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class WinVMAutomaticUpdates(BaseResourceValueCheck):
    def __init__(self) -> None:
        """
        If enabled, updates are automatically applied
        """
        name = "Ensure Windows VM enables automatic updates"
        id = "CKV_AZURE_177"
        supported_resources = ("azurerm_windows_virtual_machine", "azurerm_windows_virtual_machine_scale_set")
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self) -> str:
        return "enable_automatic_updates"


check = WinVMAutomaticUpdates()



from typing import Any

from checkov.arm.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class AppServiceMinTLSVersion(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure web app is using the latest version of TLS encryption"
        id = "CKV_AZURE_15"
        supported_resources = ("Microsoft.Web/sites",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        # https://docs.microsoft.com/en-us/azure/templates/microsoft.web/2019-08-01/sites
        return "properties/siteConfig/minTlsVersion"

    def get_expected_value(self) -> Any:
        return "1.2"


check = AppServiceMinTLSVersion()
